@@ -39,10 +39,11 @@ void frontend_set_callbacks(void);
 typedef enum {
 	STAT_SETTLEMENTS,
 	STAT_CITIES,
+	STAT_CITY_WALLS,
 	STAT_LARGEST_ARMY,
 	STAT_LONGEST_ROAD,
 	STAT_CHAPEL,
-	STAT_UNIVERSITY_OF_CATAN,
+	STAT_UNIVERSITY,
 	STAT_GOVERNORS_HOUSE,
 	STAT_LIBRARY,
 	STAT_MARKET,
@@ -53,13 +54,14 @@ typedef enum {
 
 typedef struct {
 	gchar *name;
+	gchar *style;
 	gint statistics[STAT_DEVELOPMENT + 1];
 	GList *points;		/* bonus points from special actions */
-	void *user_data;	/* used as pixmap in summary and discard list */
 } Player;
 
 typedef struct {
 	gchar *name;
+	gchar *style;
 	gint num;
 } Viewer;
 
@@ -69,6 +71,7 @@ enum callback_mode {
 	MODE_SETUP,		/* do a setup */
 	MODE_TURN,		/* your turn */
 	MODE_ROBBER,		/* place robber */
+	MODE_ROB,		/* select a building/ship to rob */
 	MODE_MONOPOLY,		/* choose monopoly resource */
 	MODE_MONOPOLY_RESPONSE,	/* chosen monopoly resource, waiting */
 	MODE_PLENTY,		/* choose year of plenty resources */
@@ -205,10 +208,16 @@ struct callbacks {
 	void (*draw_hex) (Hex * hex);
 	/* something happened to your pieces stock (ships, roads, etc.) */
 	void (*update_stock) (void);
-	/* You should move the robber or pirate, and steal if allowed */
+	/* You should move the robber or pirate */
 	void (*robber) (void);
 	/* Someone moved the robber */
 	void (*robber_moved) (Hex * old, Hex * new);
+	/* You should steal something from a building */
+	void (*steal_building) (void);
+	/* The robber placement has finished, continue normally */
+	void (*robber_done) (void);
+	/* You should steal something from a ship */
+	void (*steal_ship) (void);
 	/* Someone has been robbed.  The frontend should allow player_num to
 	 * be negative, meaning noone was robbed.  This is not implemented
 	 * yet. */
@@ -239,6 +248,8 @@ struct callbacks {
 	void (*viewer_name) (gint viewer_num, const gchar * name);
 	/* a player changed his/her name */
 	void (*player_name) (gint player_num, const gchar * name);
+	/* a player changed his/her style */
+	void (*player_style) (gint player_num, const gchar * style);
 	/* a player left the game */
 	void (*player_quit) (gint player_num);
 	/* a viewer left the game */
@@ -283,7 +294,7 @@ extern gboolean color_chat_enabled;
  * changes to the board, etc.  The frontend should NEVER touch any game
  * structures directly (except for reading). */
 void cb_connect(const gchar * server, const gchar * port,
-		const gchar * name, gboolean viewer);
+		const gchar * name, gboolean viewer, const gchar * style);
 void cb_disconnect(void);
 void cb_roll(void);
 void cb_build_road(const Edge * edge);
@@ -292,13 +303,15 @@ void cb_build_bridge(const Edge * edge);
 void cb_move_ship(const Edge * from, const Edge * to);
 void cb_build_settlement(const Node * node);
 void cb_build_city(const Node * node);
+void cb_build_city_wall(const Node * node);
 void cb_buy_develop(void);
 void cb_play_develop(int card);
 void cb_undo(void);
 void cb_maritime(gint ratio, Resource supply, Resource receive);
 void cb_domestic(const gint * supply, const gint * receive);
 void cb_end_turn(void);
-void cb_place_robber(const Hex * hex, gint victim_num);
+void cb_place_robber(const Hex * hex);
+void cb_rob(gint victim_num);
 void cb_choose_monopoly(gint resource);
 void cb_choose_plenty(gint * resources);
 void cb_trade(gint player, gint quote, const gint * supply,
@@ -309,6 +322,7 @@ void cb_delete_quote(gint num);
 void cb_end_quote(void);
 void cb_chat(const gchar * text);
 void cb_name_change(const gchar * name);
+void cb_style_change(const gchar * name);
 void cb_discard(const gint * resources);
 void cb_choose_gold(const gint * resources);
 
@@ -327,6 +341,9 @@ gint player_get_score(gint player_num);
 gint my_player_num(void);
 const gchar *my_player_name(void);
 gboolean my_player_viewer(void);
+const gchar *my_player_style(void);
+const gchar *player_get_style(gint player_num);
+void player_set_style(gint player_num, const gchar * style);
 gint num_players(void);
 gint current_player(void);
 /** Find the player or viewer with name
@@ -342,6 +359,7 @@ gint stock_num_ships(void);
 gint stock_num_bridges(void);
 gint stock_num_settlements(void);
 gint stock_num_cities(void);
+gint stock_num_city_walls(void);
 gint stock_num_develop(void);
 gint resource_asset(Resource which);
 gint resource_count(const gint * resources);
@@ -361,17 +379,16 @@ gboolean road_building_can_build_road(void);
 gboolean road_building_can_build_ship(void);
 gboolean road_building_can_build_bridge(void);
 gboolean road_building_can_finish(void);
-gboolean turn_can_undo(void);
 gboolean turn_can_build_road(void);
 gboolean turn_can_build_ship(void);
 gboolean turn_can_move_ship(void);
 gboolean turn_can_build_bridge(void);
 gboolean turn_can_build_settlement(void);
 gboolean turn_can_build_city(void);
+gboolean turn_can_build_city_wall(void);
 gboolean turn_can_trade(void);
 gboolean turn_can_finish(void);
 gboolean can_afford(const gint * cost);
-gboolean setup_can_undo(void);
 gboolean setup_can_build_road(void);
 gboolean setup_can_build_ship(void);
 gboolean setup_can_build_bridge(void);
@@ -383,6 +400,7 @@ gboolean setup_check_bridge(const Edge * edge);
 gboolean setup_check_settlement(const Node * node);
 gboolean have_ships(void);
 gboolean have_bridges(void);
+gboolean have_city_walls(void);
 const GameParams *get_game_params(void);
 int pirate_count_victims(const Hex * hex, gint * victim_list);
 int robber_count_victims(const Hex * hex, gint * victim_list);
