@@ -278,6 +278,15 @@ static void dummy_robber(void)
 }
 static void dummy_robber_moved(G_GNUC_UNUSED Hex * old,
 			       G_GNUC_UNUSED Hex * new)
+{
+};
+static void dummy_steal_building(void)
+{;
+}
+static void dummy_steal_ship(void)
+{;
+}
+static void dummy_robber_done(void)
 {;
 }
 static void dummy_player_robbed(G_GNUC_UNUSED gint robber_num,
@@ -309,6 +318,10 @@ static void dummy_player_name(G_GNUC_UNUSED gint player_num,
 			      G_GNUC_UNUSED const gchar * name)
 {;
 }
+static void dummy_player_style(G_GNUC_UNUSED gint player_num,
+			       G_GNUC_UNUSED const gchar * style)
+{;
+}
 static void dummy_player_quit(G_GNUC_UNUSED gint player_num)
 {;
 }
@@ -382,12 +395,16 @@ void client_init(void)
 	callbacks.update_stock = &dummy_update_stock;
 	callbacks.robber = &dummy_robber;
 	callbacks.robber_moved = &dummy_robber_moved;
+	callbacks.steal_building = &dummy_steal_building;
+	callbacks.steal_ship = &dummy_steal_ship;
+	callbacks.robber_done = &dummy_robber_done;
 	callbacks.player_robbed = &dummy_player_robbed;
 	callbacks.get_rolled_resources = &dummy_get_rolled_resources;
 	callbacks.new_statistics = &dummy_new_statistics;
 	callbacks.new_points = &dummy_new_points;
 	callbacks.viewer_name = &dummy_viewer_name;
 	callbacks.player_name = &dummy_player_name;
+	callbacks.player_style = &dummy_player_style;
 	callbacks.player_quit = &dummy_player_quit;
 	callbacks.viewer_quit = &dummy_viewer_quit;
 	callbacks.incoming_chat = &dummy_incoming_chat;
@@ -526,6 +543,11 @@ static gboolean check_chat_or_name(StateMachine * sm)
 		g_free(str);
 		return TRUE;
 	}
+	if (sm_recv(sm, "player %d style %S", &player_num, &str)) {
+		player_change_style(player_num, str);
+		g_free(str);
+		return TRUE;
+	}
 	return FALSE;
 }
 
@@ -548,6 +570,7 @@ static gboolean check_other_players(StateMachine * sm)
 
 	if (check_chat_or_name(sm))
 		return TRUE;
+
 	if (!sm_recv_prefix(sm, "player %d ", &player_num))
 		return FALSE;
 
@@ -645,6 +668,7 @@ static gboolean check_other_players(StateMachine * sm)
 		return TRUE;
 	}
 	if (sm_recv(sm, "must-discard %d", &discard_num)) {
+		waiting_for_network(FALSE);
 		sm_push(sm, mode_discard);
 		if (player_num == my_player_num())
 			callback_mode = MODE_DISCARD;
@@ -661,8 +685,20 @@ static gboolean check_other_players(StateMachine * sm)
 		robber_begin_move(player_num);
 		return TRUE;
 	}
-	if (sm_recv(sm, "robber %d %d", &x, &y)) {
-		robber_moved(player_num, x, y);
+	if (sm_recv(sm, "moved-robber %d %d", &x, &y)) {
+		robber_moved(player_num, x, y, FALSE);
+		return TRUE;
+	}
+	if (sm_recv(sm, "moved-pirate %d %d", &x, &y)) {
+		pirate_moved(player_num, x, y, FALSE);
+		return TRUE;
+	}
+	if (sm_recv(sm, "unmoved-robber %d %d", &x, &y)) {
+		robber_moved(player_num, x, y, TRUE);
+		return TRUE;
+	}
+	if (sm_recv(sm, "unmoved-pirate %d %d", &x, &y)) {
+		pirate_moved(player_num, x, y, TRUE);
 		return TRUE;
 	}
 	if (sm_recv(sm, "stole from %d", &victim_num)) {
@@ -844,6 +880,7 @@ gboolean mode_start(StateMachine * sm, gint event)
 		g_free(version);
 		player_set_my_num(player_num);
 		player_set_total_num(total_num);
+		sm_send(sm, "style %s\n", requested_style);
 		sm_send(sm, "players\n");
 		sm_goto(sm, mode_players);
 
@@ -1077,8 +1114,7 @@ static gboolean mode_load_gameinfo(StateMachine * sm, gint event)
 		}
 		if (puniv) {
 			player_modify_statistic(my_player_num(),
-						STAT_UNIVERSITY_OF_CATAN,
-						1);
+						STAT_UNIVERSITY, 1);
 		}
 		if (pgov) {
 			player_modify_statistic(my_player_num(),
@@ -1124,9 +1160,7 @@ static gboolean mode_load_gameinfo(StateMachine * sm, gint event)
 			player_modify_statistic(opnum, STAT_CHAPEL, 1);
 		}
 		if (puniv) {
-			player_modify_statistic(opnum,
-						STAT_UNIVERSITY_OF_CATAN,
-						1);
+			player_modify_statistic(opnum, STAT_UNIVERSITY, 1);
 		}
 		if (pgov) {
 			player_modify_statistic(opnum,
@@ -1176,6 +1210,10 @@ static gboolean mode_load_gameinfo(StateMachine * sm, gint event)
 		player_build_add(owner, BUILD_CITY, x, y, pos, FALSE);
 		return TRUE;
 	}
+	if (sm_recv(sm, "W%d,%d,%d,%d", &x, &y, &pos, &owner)) {
+		player_build_add(owner, BUILD_CITY_WALL, x, y, pos, FALSE);
+		return TRUE;
+	}
 	if (sm_recv(sm, "R%d,%d,%d,%d", &x, &y, &pos, &owner)) {
 		player_build_add(owner, BUILD_ROAD, x, y, pos, FALSE);
 		return TRUE;
@@ -1366,7 +1404,7 @@ static gboolean mode_setup(StateMachine * sm, gint event)
  */
 static gboolean mode_idle(StateMachine * sm, gint event)
 {
-	gint num, player_num, x, y, backwards;
+	gint num, player_num, backwards;
 	gint they_supply[NO_RESOURCE];
 	gint they_receive[NO_RESOURCE];
 
@@ -1396,16 +1434,6 @@ static gboolean mode_idle(StateMachine * sm, gint event)
 			sm_push(sm, mode_turn);
 			return TRUE;
 		}
-		if (sm_recv(sm, "player %d moved-robber %d %d",
-			    &player_num, &x, &y)) {
-			robber_moved(player_num, x, y);
-			return TRUE;
-		}
-		if (sm_recv(sm, "player %d moved-pirate %d %d",
-			    &player_num, &x, &y)) {
-			pirate_moved(player_num, x, y);
-			return TRUE;
-		}
 		if (sm_recv
 		    (sm,
 		     "player %d domestic-trade call supply %R receive %R",
@@ -1428,30 +1456,105 @@ static gboolean mode_idle(StateMachine * sm, gint event)
  * Nested state machine for robber handling
  */
 
+/* Get user to steal from a building
+ */
+static gboolean mode_robber_steal_building(StateMachine * sm, gint event)
+{
+	sm_state_name(sm, "mode_robber_steal_building");
+	switch (event) {
+	case SM_ENTER:
+		callback_mode = MODE_ROB;
+		callbacks.
+		    instructions(_("Select the building to steal from."));
+		callbacks.steal_building();
+		break;
+	case SM_RECV:
+		if (check_other_players(sm))
+			return TRUE;
+		break;
+	default:
+		break;
+	}
+	return FALSE;
+}
+
+/* Get user to steal from a ship
+ */
+static gboolean mode_robber_steal_ship(StateMachine * sm, gint event)
+{
+	sm_state_name(sm, "mode_robber_steal_ship");
+	switch (event) {
+	case SM_ENTER:
+		callback_mode = MODE_ROB;
+		callbacks.
+		    instructions(_("Select the ship to steal from."));
+		callbacks.steal_ship();
+		break;
+	case SM_RECV:
+		if (check_other_players(sm))
+			return TRUE;
+		break;
+	default:
+		break;
+	}
+	return FALSE;
+}
+
 /* Handle response to move robber
  */
-gboolean mode_robber_response(StateMachine * sm, gint event)
+gboolean mode_robber_move_response(StateMachine * sm, gint event)
 {
 	gint x, y;
 
+	sm_state_name(sm, "mode_robber_move_response");
+	switch (event) {
+	case SM_ENTER:
+		waiting_for_network(TRUE);
+		break;
+	case SM_RECV:
+		if (sm_recv(sm, "robber-done")) {
+			waiting_for_network(FALSE);
+			sm_multipop(sm, 2);
+			callbacks.robber_done();
+			return TRUE;
+		}
+		if (sm_recv(sm, "rob %d %d", &x, &y)) {
+			Hex *hex;
+			waiting_for_network(FALSE);
+			hex = map_hex(map, x, y);
+			if (hex->terrain == SEA_TERRAIN)
+				sm_push(sm, mode_robber_steal_ship);
+			else
+				sm_push(sm, mode_robber_steal_building);
+			return TRUE;
+		}
+		if (check_other_players(sm))
+			return TRUE;
+		break;
+	}
+	return FALSE;
+}
+
+/* Wait for server to say robber-done */
+gboolean mode_robber_response(StateMachine * sm, gint event)
+{
 	sm_state_name(sm, "mode_robber_response");
 	switch (event) {
 	case SM_ENTER:
 		waiting_for_network(TRUE);
 		break;
 	case SM_RECV:
-		if (sm_recv(sm, "moved-robber %d %d", &x, &y)) {
-			robber_moved(my_player_num(), x, y);
+		if (sm_recv(sm, "robber-done")) {
 			waiting_for_network(FALSE);
-			/* pop back to parent's parent if robber is moved */
-			sm_multipop(sm, 2);
-			return TRUE;
-		}
-		if (sm_recv(sm, "moved-pirate %d %d", &x, &y)) {
-			pirate_moved(my_player_num(), x, y);
-			waiting_for_network(FALSE);
-			/* pop back to parent's parent if pirate is moved */
-			sm_multipop(sm, 2);
+			sm_stack_dump(sm);
+			/* current state is response
+			 * parent is steal
+			 * parent is move_response
+			 * parent is mode_robber
+			 * all four must be popped.
+			 */
+			sm_multipop(sm, 4);
+			callbacks.robber_done();
 			return TRUE;
 		}
 		if (check_other_players(sm))
@@ -1494,10 +1597,11 @@ static gboolean mode_wait_for_robber(StateMachine * sm, gint event)
 	sm_state_name(sm, "mode_wait_for_robber");
 	switch (event) {
 	case SM_ENTER:
-		callbacks.instructions(_("Place the robber"));
+		waiting_for_network(TRUE);
 		break;
 	case SM_RECV:
 		if (sm_recv(sm, "you-are-robber")) {
+			waiting_for_network(FALSE);
 			sm_goto(sm, mode_robber);
 			return TRUE;
 		}
@@ -1865,6 +1969,17 @@ gboolean mode_undo_response(StateMachine * sm, gint event)
 			sm_pop(sm);
 			return TRUE;
 		}
+		if (sm_recv(sm, "undo-robber")) {
+			waiting_for_network(FALSE);
+			/* current state is undo-response
+			 * parent is steal
+			 * parent is move_response
+			 * parent is mode_robber
+			 * the first three must be popped.
+			 */
+			sm_multipop(sm, 3);
+			return TRUE;
+		}
 		if (check_other_players(sm))
 			return TRUE;
 		break;
@@ -2396,6 +2511,9 @@ static void recover_from_disconnect(StateMachine * sm,
 			sm_push_noenter(sm, mode_turn_rolled);
 			sm_push_noenter(sm, mode_wait_for_robber);
 		}
+		/* Allow gui to fill previous_state when entering
+		 * mode_discard.  */
+		callbacks.turn();
 		sm_push(sm, mode_discard);
 	} else if (strcmp(rinfo->prevstate, "MONOPOLY") == 0) {
 		sm_goto_noenter(sm, mode_idle);
