@@ -44,14 +44,22 @@ void player_reset(void)
 	gint i, idx;
 
 	/* remove all viewers */
-	while (viewers != NULL)
-		viewers = g_list_remove(viewers, viewers->data);
+	while (viewers != NULL) {
+		Viewer *viewer = viewers->data;
+		g_free(viewer->name);
+		g_free(viewer->style);
+		viewers = g_list_remove(viewers, viewer);
+	}
 	/* free player's memory */
 	for (i = 0; i < MAX_PLAYERS; ++i) {
 		if (players[i].name != NULL) {
 			g_free(players[i].name);
 			players[i].name = NULL;
 		}
+		if (players[i].style != NULL) {
+			g_free(players[i].style);
+			players[i].style = NULL;
+		}
 		while (players[i].points != NULL) {
 			Points *points = players[i].points->data;
 			players[i].points =
@@ -195,6 +203,7 @@ void player_change_name(gint player_num, const gchar * name)
 			viewers = g_list_prepend(viewers, viewer);
 			viewer->num = player_num;
 			viewer->name = NULL;
+			viewer->style = g_strdup("square");
 			old_name = NULL;
 		} else {
 			old_name = viewer->name;
@@ -226,6 +235,12 @@ void player_change_name(gint player_num, const gchar * name)
 	callbacks.player_name(player_num, name);
 }
 
+void player_change_style(gint player_num, const gchar * style)
+{
+	player_set_style(player_num, style);
+	callbacks.player_style(player_num, style);
+}
+
 void player_has_quit(gint player_num)
 {
 	Player *player;
@@ -243,6 +258,7 @@ void player_has_quit(gint player_num)
 		callbacks.viewer_quit(player_num);
 		viewer = viewer_get(player_num);
 		g_free(viewer->name);
+		g_free(viewer->style);
 		viewers = g_list_remove(viewers, viewer);
 		return;
 	}
@@ -507,6 +523,21 @@ void player_build_add(gint player_num,
 		if (player_num == my_player_num())
 			stock_use_city();
 		break;
+	case BUILD_CITY_WALL:
+		node = map_node(map, x, y, pos);
+		node->city_wall = TRUE;
+		node->owner = player_num;
+		callbacks.draw_node(node);
+		if (log_changes) {
+			log_message(MSG_BUILD,
+				    _("%s built a city wall.\n"),
+				    player_name(player_num, TRUE));
+		}
+		player_modify_statistic(player_num, STAT_CITY_WALLS, 1);
+		if (player_num == my_player_num())
+			stock_use_city_wall();
+		break;
+
 	case BUILD_NONE:
 		log_message(MSG_ERROR,
 			    _
@@ -585,6 +616,18 @@ void player_build_remove(gint player_num,
 			stock_replace_city();
 		}
 		break;
+	case BUILD_CITY_WALL:
+		node = map_node(map, x, y, pos);
+		node->city_wall = FALSE;
+		node->owner = player_num;
+		callbacks.draw_node(node);
+		log_message(MSG_BUILD, _("%s removed a city wall.\n"),
+			    player_name(player_num, TRUE));
+		player_modify_statistic(player_num, STAT_CITY_WALLS, -1);
+		if (player_num == my_player_num())
+			stock_replace_city_wall();
+		break;
+
 	case BUILD_NONE:
 		log_message(MSG_ERROR,
 			    _
@@ -708,6 +751,34 @@ void player_take_point(gint player_num, gint id, gint old_owner)
 		    point->name, player->name);
 }
 
+void player_set_style(gint player_num, const gchar * style)
+{
+	if (player_num >= num_total_players) {
+		Viewer *viewer = viewer_get(player_num);
+		if (viewer->style)
+			g_free(viewer->style);
+		viewer->style = g_strdup(style);
+	} else if (player_num >= 0) {
+		Player *player = player_get(player_num);
+		if (player->style)
+			g_free(player->style);
+		player->style = g_strdup(style);
+	}
+}
+
+const gchar *player_get_style(gint player_num)
+{
+	const gchar *style = NULL;
+	if (player_num >= num_total_players) {
+		Viewer *viewer = viewer_get(player_num);
+		style = viewer->style;
+	} else if (player_num >= 0) {
+		Player *player = player_get(player_num);
+		style = player->style;
+	}
+	return style == NULL ? "square" : style;
+}
+
 gint current_player(void)
 {
 	return turn_player;
@@ -723,6 +794,11 @@ gboolean my_player_viewer(void)
 	return player_is_viewer(my_player_num());
 }
 
+const gchar *my_player_style(void)
+{
+	return player_get_style(my_player_num());
+}
+
 gint find_player_by_name(const gchar * name)
 {
 	gint i;
