@@ -47,6 +47,7 @@ typedef enum {
 	GUI_BRIDGE,
 	GUI_SETTLEMENT,
 	GUI_CITY,
+	GUI_CITY_WALL,
 	GUI_BUY_DEVELOP,
 	GUI_PLAY_DEVELOP,
 	GUI_MONOPOLY,
@@ -105,6 +106,7 @@ void frontend_new_statistics(gint player_num, StatisticType type,
 void frontend_new_points(gint player_num, Points * points, gboolean added);
 void frontend_viewer_name(gint viewer_num, const gchar * name);
 void frontend_player_name(gint player_num, const gchar * name);
+void frontend_player_style(gint player_num, const gchar * style);
 void frontend_player_quit(gint player_num);
 void frontend_viewer_quit(gint player_num);
 void frontend_disconnect(void);
@@ -144,6 +146,9 @@ void frontend_played_develop(gint player_num, gint card_idx,
 			     DevelType type);
 void frontend_resource_change(Resource type, gint new_amount);
 void frontend_robber(void);
+void frontend_steal_building(void);
+void frontend_steal_ship(void);
+void frontend_robber_done(void);
 void frontend_game_over(gint player, gint points);
 
 /* connect.c */
@@ -151,10 +156,12 @@ const gchar *connect_get_server(void);
 const gchar *connect_get_port(void);
 const gchar *connect_get_name(void);
 gboolean connect_get_viewer(void);
+const gchar *connect_get_style(void);
 void connect_set_server(const gchar * server);
 void connect_set_port(const gchar * port);
 void connect_set_name(const gchar * name);
 void connect_set_viewer(gboolean viewer);
+void connect_set_style(const gchar * style);
 void connect_set_meta_server(const gchar * meta_server);
 void connect_create_dlg(void);
 
@@ -276,6 +283,8 @@ void chat_set_grab_focus_on_update(gboolean grab);
 void chat_set_focus(void);
 /** A player/viewer has changed his name */
 void chat_player_name(gint player_num, const gchar * name);
+/** A player/viewer has changed his style */
+void chat_player_style(gint player_num);
 /** A player has quit */
 void chat_player_quit(gint player_num);
 /** A viewer has quit */
@@ -314,6 +323,7 @@ GtkWidget *gameover_create_dlg(gint player_num, gint num_points);
 #define PIONEERS_PIXMAP_BRIDGE "pioneers/bridge.png"
 #define PIONEERS_PIXMAP_SETTLEMENT "pioneers/settlement.png"
 #define PIONEERS_PIXMAP_CITY "pioneers/city.png"
+#define PIONEERS_PIXMAP_CITY_WALL "pioneers/city_wall.png"
 #define PIONEERS_PIXMAP_DEVELOP "pioneers/develop.png"
 #define PIONEERS_PIXMAP_FINISH "pioneers/finish.png"
 
