@@ -71,6 +71,9 @@ static GtkWidget *app_bar;
 static GtkWidget *net_status;
 static GtkWidget *vp_target_status;
 
+static GtkWidget *main_paned;	/* Horizontal for 16:9, Vertical for 4:3 mode */
+static GtkWidget *chat_panel = NULL;	/* Panel for chat, placed below or to the right */
+
 static GtkUIManager *ui_manager = NULL;	/* The manager of the GtkActions */
 static GtkWidget *toolbar = NULL;	/* The toolbar */
 
@@ -91,6 +94,7 @@ static const gchar *pioneers_pixmaps[] = {
 	PIONEERS_PIXMAP_BRIDGE,
 	PIONEERS_PIXMAP_SETTLEMENT,
 	PIONEERS_PIXMAP_CITY,
+	PIONEERS_PIXMAP_CITY_WALL,
 	PIONEERS_PIXMAP_DEVELOP,
 	PIONEERS_PIXMAP_FINISH
 };
@@ -188,6 +192,11 @@ static void buy_development_cb(void)
 	route_gui_event(GUI_BUY_DEVELOP);
 }
 
+static void build_city_wall_cb(void)
+{
+	route_gui_event(GUI_CITY_WALL);
+}
+
 static void showhide_toolbar_cb(void);
 static void preferences_cb(void);
 
@@ -242,6 +251,8 @@ static GtkActionEntry entries[] = {
 	 N_("Build a city"), build_city_cb},
 	{"BuyDevelopment", PIONEERS_PIXMAP_DEVELOP, N_("Develop"), "F11",
 	 N_("Buy a development card"), buy_development_cb},
+	{"BuildCityWall", PIONEERS_PIXMAP_CITY_WALL, N_("City Wall"), NULL,
+	 N_("Build a city wall"), build_city_wall_cb},
 
 	{"SettingsMenu", NULL, N_("_Settings"), NULL, NULL, NULL},
 	{"Preferences", GTK_STOCK_PREFERENCES, N_("Prefere_nces"), NULL,
@@ -294,6 +305,7 @@ static const char *ui_description =
 "      <menuitem action='BuildSettlement'/>"
 "      <menuitem action='BuildCity'/>"
 "      <menuitem action='BuyDevelopment'/>"
+"      <menuitem action='BuildCityWall'/>"
 "    </menu>"
 "    <menu action='SettingsMenu'>"
 "      <menuitem action='ShowHideToolbar'/>"
@@ -318,6 +330,7 @@ static const char *ui_description =
 "    <toolitem action='BuildSettlement'/>"
 "    <toolitem action='BuildCity'/>"
 "    <toolitem action='BuyDevelopment'/>"
+"    <toolitem action='BuildCityWall'/>"
 "  </toolbar>"
 "</ui>";
 /* *INDENT-ON* */
@@ -659,6 +672,49 @@ static GtkWidget *build_develop_panel(void)
 	return develop_notebook;
 }
 
+static gboolean get_16_9_layout(void)
+{
+	GtkWidget *paned;
+
+	g_return_val_if_fail(main_paned != NULL, FALSE);
+	g_return_val_if_fail(chat_panel != NULL, FALSE);
+
+	paned = gtk_paned_get_child1(GTK_PANED(main_paned));
+	if (gtk_widget_get_parent(chat_panel) == paned)
+		return FALSE;
+	return TRUE;
+}
+
+static void set_16_9_layout(gboolean layout_16_9)
+{
+	GtkWidget *paned;
+	gboolean can_remove;
+
+	g_return_if_fail(main_paned != NULL);
+	g_return_if_fail(chat_panel != NULL);
+
+	paned = gtk_paned_get_child1(GTK_PANED(main_paned));
+
+	/* Increase reference count, otherwise it will be destroyed */
+	g_object_ref(chat_panel);
+
+	/* Initially the widget has no parent, and cannot be removed */
+	can_remove = gtk_widget_get_parent(chat_panel) != NULL;
+
+	if (layout_16_9) {
+		if (can_remove)
+			gtk_container_remove(GTK_CONTAINER(paned),
+					     chat_panel);
+		gtk_container_add(GTK_CONTAINER(main_paned), chat_panel);
+	} else {
+		if (can_remove)
+			gtk_container_remove(GTK_CONTAINER(main_paned),
+					     chat_panel);
+		gtk_container_add(GTK_CONTAINER(paned), chat_panel);
+	}
+	g_object_unref(chat_panel);
+}
+
 static GtkWidget *build_main_interface(void)
 {
 	GtkWidget *vbox;
@@ -682,29 +738,36 @@ static GtkWidget *build_main_interface(void)
 	gtk_box_pack_start(GTK_BOX(vbox),
 			   player_build_summary(), TRUE, TRUE, 0);
 
+	main_paned = gtk_hpaned_new();
+	gtk_widget_show(main_paned);
+
 	vpaned = gtk_vpaned_new();
 	gtk_widget_show(vpaned);
-	gtk_paned_pack2(GTK_PANED(hpaned), vpaned, TRUE, TRUE);
+
+	gtk_paned_pack1(GTK_PANED(main_paned), vpaned, TRUE, TRUE);
 
 	gtk_paned_pack1(GTK_PANED(vpaned), build_map_panel(), TRUE, TRUE);
 
-	vbox = gtk_vbox_new(FALSE, 0);
-	gtk_widget_show(vbox);
+	chat_panel = gtk_vbox_new(FALSE, 0);
+	gtk_widget_show(chat_panel);
 
 	panel = chat_build_panel();
 	frontend_gui_register(panel, GUI_DISCONNECT, NULL);
-	gtk_box_pack_start(GTK_BOX(vbox), panel, FALSE, TRUE, 0);
-	gtk_box_pack_start(GTK_BOX(vbox),
+	gtk_box_pack_start(GTK_BOX(chat_panel), panel, FALSE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(chat_panel),
 			   build_messages_panel(), TRUE, TRUE, 0);
 
-	gtk_paned_pack2(GTK_PANED(vpaned), vbox, FALSE, TRUE);
+	set_16_9_layout(config_get_int_with_default
+			("settings/layout_16_9", FALSE));
 
+	gtk_paned_pack2(GTK_PANED(hpaned), main_paned, TRUE, TRUE);
 	return hpaned;
 }
 
 static void quit_cb(G_GNUC_UNUSED GtkWidget * widget,
 		    G_GNUC_UNUSED void *data)
 {
+	guimap_delete(gmap);
 	gtk_main_quit();
 }
 
@@ -763,6 +826,14 @@ static void announce_player_cb(GtkToggleButton * widget,
 	set_announce_player(announce_player);
 }
 
+static void toggle_16_9_cb(GtkToggleButton * widget,
+			   G_GNUC_UNUSED gpointer user_data)
+{
+	gboolean layout_16_9 = gtk_toggle_button_get_active(widget);
+	config_set_int("settings/layout_16_9", layout_16_9);
+	set_16_9_layout(layout_16_9);
+}
+
 static void showhide_toolbar_cb(void)
 {
 	gui_set_toolbar_visible();
@@ -943,6 +1014,23 @@ static void preferences_cb(void)
 			     ("Make a sound when a new player or viewer enters the game"),
 			     NULL);
 	row++;
+
+	/* Label for the option to use the 16:9 layout. */
+	widget = gtk_check_button_new_with_label(_("Use 16:9 layout"));
+	gtk_widget_show(widget);
+	gtk_table_attach_defaults(GTK_TABLE(layout), widget,
+				  0, 2, row, row + 1);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget),
+				     get_16_9_layout());
+	g_signal_connect(G_OBJECT(widget), "toggled",
+			 G_CALLBACK(toggle_16_9_cb), NULL);
+	/* Tooltip for 16:9 option. */
+	gtk_tooltips_set_tip(tooltips, widget,
+			     _
+			     ("Use a 16:9 friendly layout for the window"),
+			     NULL);
+	row++;
+
 }
 
 static void help_about_cb(void)
@@ -1036,6 +1124,9 @@ static GtkAction *getAction(GuiEvent id)
 	case GUI_BUY_DEVELOP:
 		path = "ActionsMenu/BuyDevelopment";
 		break;
+	case GUI_CITY_WALL:
+		path = "ActionsMenu/BuildCityWall";
+		break;
 	default:
 		break;
 	};
@@ -1084,46 +1175,59 @@ static void gui_toolbar_show_accelerators(gboolean show_accelerators)
 	for (i = 0; i < n; i++) {
 		GtkToolItem *ti;
 		GtkToolButton *tbtn;
-		const gchar *text;
+		gchar *text;
 		gint j;
 
 		ti = gtk_toolbar_get_nth_item(tb, i);
 		tbtn = GTK_TOOL_BUTTON(ti);
 		g_assert(tbtn != NULL);
-		text = gtk_tool_button_get_label(tbtn);
+		if (gtk_major_version == 2 && gtk_minor_version == 10) {
+			/* Work around a gtk+ 2.10 bug (#434261) that
+			 * mishandles strings like (Fn) in labels.
+			 */
+			/** @todo BW 2007-04-29 Remove this when gtk 2.10
+			 * is no longer supported. */
+			gtk_tool_button_set_use_underline(tbtn, FALSE);
+		}
+		text = g_strdup(gtk_tool_button_get_label(tbtn));
+		if (strchr(text, '\n'))
+			*strchr(text, '\n') = '\0';
 		/* Find the matching entry */
 		for (j = 0; j < G_N_ELEMENTS(entries); j++) {
-			if (g_str_has_prefix(text, _(entries[j].label))) {
+			if (strcmp(text, _(entries[j].label)) == 0) {
 				if (show_accelerators) {
-					gchar *accelerator_text;
 					gchar *label;
-					/*
-					   guint accelerator_key;
-					   GdkModifierType accelerator_mods;
 
-					   gtk_accelerator_parse(entries[j].accelerator, &accelerator_key, &accelerator_mods);
-					 */
-					accelerator_text =
-					    g_strdup(entries[j].
-						     accelerator);
-					/** @todo RC 2005-07-10 When Gtk+-2.6 
-					 * is required, use the line below
-					 * instead.
-					 * For now it is not a real problem,
-					 * since the shortcuts are function
-					 * keys (Fn)
-					 */
-					/*
-					   accelerator_text = gtk_accelerator_get_label(accelerator_key, accelerator_mods);
-					 */
-					label =
-					    g_strdup_printf("%s\n(%s)",
-							    text,
-							    accelerator_text);
+					if (entries[j].accelerator == NULL
+					    || strlen(entries[j].
+						      accelerator)
+					    == 0)
+						label =
+						    g_strdup_printf("%s\n",
+								    text);
+					else {
+						gchar *accelerator_text;
+						guint accelerator_key;
+						GdkModifierType
+						    accelerator_mods;
+						gtk_accelerator_parse
+						    (entries[j].
+						     accelerator,
+						     &accelerator_key,
+						     &accelerator_mods);
+						accelerator_text =
+						    gtk_accelerator_get_label
+						    (accelerator_key,
+						     accelerator_mods);
+						label =
+						    g_strdup_printf
+						    ("%s\n(%s)", text,
+						     accelerator_text);
+						g_free(accelerator_text);
+					}
 					gtk_tool_button_set_label(tbtn,
 								  label);
 					g_free(label);
-					g_free(accelerator_text);
 				} else {
 					gtk_tool_button_set_label(tbtn,
 								  _(entries
@@ -1133,6 +1237,7 @@ static void gui_toolbar_show_accelerators(gboolean show_accelerators)
 				break;
 			}
 		}
+		g_free(text);
 	}
 	config_set_int("settings/toolbar_show_accelerators",
 		       toolbar_show_accelerators);
@@ -1186,6 +1291,9 @@ void gui_set_game_params(const GameParams * params)
 	/* In theory, it is possible to play a game without cities */
 	gui_toolbar_show_button("BuildCity",
 				params->num_build_type[BUILD_CITY] > 0);
+	gui_toolbar_show_button("BuildCityWall",
+				params->num_build_type[BUILD_CITY_WALL] >
+				0);
 
 	identity_draw();
 
@@ -1453,6 +1561,8 @@ GtkWidget *gui_build_interface(void)
 	frontend_gui_register_action(getAction(GUI_CITY), GUI_CITY);
 	frontend_gui_register_action(getAction(GUI_BUY_DEVELOP),
 				     GUI_BUY_DEVELOP);
+	frontend_gui_register_action(getAction(GUI_CITY_WALL),
+				     GUI_CITY_WALL);
 #if 0
 	frontend_gui_register_destroy(gtk_ui_manager_get_action
 				      (manager, "GameQuit"), GUI_QUIT);
