@@ -24,9 +24,9 @@
 
 #include "guimap.h"
 
-gint color_messages_enabled;
 void set_color_summary(gboolean flag);
-gint legend_page_enabled;
+gboolean get_announce_player(void);
+void set_announce_player(gboolean announce);
 
 GtkWidget *gui_get_dialog_button(GtkDialog * dlg, gint button);
 
@@ -50,7 +50,7 @@ void gui_prompt_hide(void);
 void gui_cursor_none(void);
 void gui_cursor_set(CursorType type,
 		    CheckFunc check_func, SelectFunc select_func,
-		    const MapElement * user_data);
+		    CancelFunc cancel_func, const MapElement * user_data);
 void gui_draw_hex(const Hex * hex);
 void gui_draw_edge(const Edge * edge);
 void gui_draw_node(const Node * node);
@@ -70,18 +70,19 @@ extern GtkWidget *app_window;	/* main application window */
 /* gui states */
 typedef void (*GuiState) (GuiEvent event);
 
-#ifdef DEBUG
-#define set_gui_state(A) \
-		{ g_print("New GUI_state: %s\n", #A); \
-		set_gui_state_nomacro(A); }
-#else
-#define set_gui_state(A) \
-		set_gui_state_nomacro(A)
-#endif
+#define set_gui_state(A) do \
+		{ debug("New GUI_state: %s\n", #A); \
+		set_gui_state_nomacro(A); } while (0)
 void set_gui_state_nomacro(GuiState state);
 
 GuiState get_gui_state(void);
 void route_gui_event(GuiEvent event);
 
 void gui_rules_register_callback(GCallback callback);
+
+void gui_get_resource_pixmap(gint idx, GdkPixmap ** p, GdkBitmap ** b,
+			     GdkGC ** gcp, GdkGC ** gcb);
+gint gui_get_resource_pixmap_res(void);
+void gui_set_show_no_setup_nodes(gboolean show);
+
 #endif
