@@ -33,6 +33,7 @@ static void build_bridge_cb(MapElement edge, MapElement extra);
 static void move_ship_cb(MapElement edge, MapElement extra);
 static void build_settlement_cb(MapElement node, MapElement extra);
 static void build_city_cb(MapElement node, MapElement extra);
+static void build_city_wall_cb(MapElement node, MapElement extra);
 
 static void dummy_state(G_GNUC_UNUSED GuiEvent event)
 {
@@ -112,6 +113,11 @@ void build_city_cb(MapElement node, G_GNUC_UNUSED MapElement extra)
 	cb_build_city(node.node);
 }
 
+void build_city_wall_cb(MapElement node, G_GNUC_UNUSED MapElement extra)
+{
+	cb_build_city_wall(node.node);
+}
+
 /* trade */
 static void frontend_state_trade(GuiEvent event)
 {
@@ -288,6 +294,12 @@ static gboolean check_city(MapElement element, gint owner,
 	return can_settlement_be_upgraded(element.node, owner);
 }
 
+static gboolean check_city_wall(MapElement element, gint owner,
+				G_GNUC_UNUSED MapElement extra)
+{
+	return can_city_wall_be_built(element.node, owner);
+}
+
 /* turn */
 static void frontend_state_turn(GuiEvent event)
 {
@@ -302,6 +314,8 @@ static void frontend_state_turn(GuiEvent event)
 		frontend_gui_check(GUI_SETTLEMENT,
 				   turn_can_build_settlement());
 		frontend_gui_check(GUI_CITY, turn_can_build_city());
+		frontend_gui_check(GUI_CITY_WALL,
+				   turn_can_build_city_wall());
 		frontend_gui_check(GUI_TRADE, turn_can_trade());
 		frontend_gui_check(GUI_PLAY_DEVELOP,
 				   can_play_develop(develop_current_idx
@@ -309,22 +323,21 @@ static void frontend_state_turn(GuiEvent event)
 		frontend_gui_check(GUI_BUY_DEVELOP, can_buy_develop());
 		frontend_gui_check(GUI_FINISH, have_rolled_dice());
 
-		guimap_start_single_click_build(turn_can_build_road(),
-						check_road, build_road_cb,
-						turn_can_build_ship(),
-						check_ship, build_ship_cb,
-						turn_can_build_bridge(),
-						check_bridge,
-						build_bridge_cb,
-						turn_can_build_settlement
-						(), check_settlement,
-						build_settlement_cb,
-						turn_can_build_city(),
-						check_city, build_city_cb,
-						turn_can_move_ship(),
-						check_ship_move,
-						move_ship_cb,
-						cancel_move_ship_cb);
+		guimap_single_click_set_functions(check_road,
+						  build_road_cb,
+						  check_ship,
+						  build_ship_cb,
+						  check_bridge,
+						  build_bridge_cb,
+						  check_settlement,
+						  build_settlement_cb,
+						  check_city,
+						  build_city_cb,
+						  check_city_wall,
+						  build_city_wall_cb,
+						  check_ship_move,
+						  move_ship_cb,
+						  cancel_move_ship_cb);
 		break;
 	case GUI_ROLL:
 		cb_roll();
@@ -356,6 +369,10 @@ static void frontend_state_turn(GuiEvent event)
 		gui_cursor_set(CITY_CURSOR, check_city, build_city_cb,
 			       NULL, NULL);
 		return;
+	case GUI_CITY_WALL:
+		gui_cursor_set(CITY_WALL_CURSOR, check_city_wall,
+			       build_city_wall_cb, NULL, NULL);
+		return;
 	case GUI_TRADE:
 		trade_begin();
 		set_gui_state(frontend_state_trade);
@@ -405,13 +422,15 @@ static void frontend_state_roadbuilding(GuiEvent event)
 		frontend_gui_check(GUI_BRIDGE,
 				   road_building_can_build_bridge());
 		frontend_gui_check(GUI_FINISH, road_building_can_finish());
-		guimap_start_single_click_build
-		    (road_building_can_build_road(), check_road,
-		     build_road_cb, road_building_can_build_ship(),
-		     check_ship, build_ship_cb,
-		     road_building_can_build_bridge(), check_bridge,
-		     build_bridge_cb, FALSE, NULL, NULL, FALSE, NULL,
-		     NULL, FALSE, NULL, NULL, NULL);
+		guimap_single_click_set_functions(check_road,
+						  build_road_cb,
+						  check_ship,
+						  build_ship_cb,
+						  check_bridge,
+						  build_bridge_cb,
+						  NULL, NULL, NULL,
+						  NULL, NULL, NULL,
+						  NULL, NULL, NULL);
 		break;
 	case GUI_UNDO:
 		cb_undo();
@@ -631,24 +650,30 @@ void frontend_rolled_dice(gint die1, gint die2, gint player_num)
 	frontend_gui_update();
 }
 
-static void place_robber(const Hex * hex, gint victim)
+static void frontend_state_robber(GuiEvent event)
 {
-	cb_place_robber(hex, victim);
-	robber_busy = FALSE;
-	if (previous_state != dummy_state) {
-		set_gui_state(previous_state);
-		previous_state = dummy_state;
+	switch (event) {
+	case GUI_UPDATE:
+		frontend_gui_check(GUI_UNDO, callback_mode == MODE_ROB);
+		break;
+	case GUI_UNDO:
+		cb_undo();
+		/* restart robber placement.  */
+		frontend_robber();
+		break;
+	default:
+		break;
 	}
 }
 
-static void rob_building(MapElement node, MapElement hex)
+static void rob_building(MapElement node, G_GNUC_UNUSED MapElement extra)
 {
-	place_robber(hex.hex, node.node->owner);
+	cb_rob(node.node->owner);
 }
 
-static void rob_edge(MapElement edge, MapElement hex)
+static void rob_edge(MapElement edge, G_GNUC_UNUSED MapElement extra)
 {
-	place_robber(hex.hex, edge.edge->owner);
+	cb_rob(edge.edge->owner);
 }
 
 /* Return TRUE if the node can be robbed. */
@@ -702,45 +727,37 @@ static gboolean can_edge_be_robbed(MapElement edge,
 static void place_robber_or_pirate_cb(MapElement hex,
 				      G_GNUC_UNUSED MapElement extra)
 {
-	if (hex.hex->terrain == SEA_TERRAIN) {
-		gint victim_list[G_N_ELEMENTS(hex.hex->edges)];
-		switch (pirate_count_victims(hex.hex, victim_list)) {
-		case 0:
-			place_robber(hex.hex, -1);
-			break;
-		case 1:
-			place_robber(hex.hex, victim_list[0]);
-			break;
-		default:
-			gui_cursor_set(STEAL_SHIP_CURSOR,
-				       can_edge_be_robbed, rob_edge, NULL,
-				       &hex);
-			gui_set_instructions(_
-					     ("Select the ship to steal from."));
-			gui_prompt_show(_
-					("Select the ship to steal from"));
-			break;
-		}
-	} else {
-		gint victim_list[G_N_ELEMENTS(hex.hex->nodes)];
-		switch (robber_count_victims(hex.hex, victim_list)) {
-		case 0:
-			place_robber(hex.hex, -1);
-			break;
-		case 1:
-			place_robber(hex.hex, victim_list[0]);
-			break;
-		default:
-			gui_cursor_set(STEAL_BUILDING_CURSOR,
-				       can_building_be_robbed,
-				       rob_building, NULL, &hex);
-			gui_set_instructions(_
-					     ("Select the building to steal from."));
-			gui_prompt_show(_
-					("Select the building to steal from"));
-			break;
-		}
+	cb_place_robber(hex.hex);
+}
+
+void frontend_steal_ship(void)
+{
+	MapElement hex;
+	hex.hex = map_pirate_hex(get_map());
+
+	gui_cursor_set(STEAL_SHIP_CURSOR, can_edge_be_robbed, rob_edge,
+		       NULL, &hex);
+	gui_prompt_show(_("Select the ship to steal from"));
+}
+
+void frontend_steal_building(void)
+{
+	MapElement hex;
+	hex.hex = map_robber_hex(get_map());
+
+	gui_cursor_set(STEAL_BUILDING_CURSOR, can_building_be_robbed,
+		       rob_building, NULL, &hex);
+	gui_prompt_show(_("Select the building to steal from"));
+}
+
+void frontend_robber_done(void)
+{
+	robber_busy = FALSE;
+	if (previous_state != dummy_state) {
+		set_gui_state(previous_state);
+		previous_state = dummy_state;
 	}
+	gui_prompt_hide();
 }
 
 static gboolean check_move_robber_or_pirate(MapElement element,
@@ -752,14 +769,15 @@ static gboolean check_move_robber_or_pirate(MapElement element,
 
 void frontend_robber(void)
 {
-	if (robber_busy)
-		return;
-	robber_busy = TRUE;
-	g_assert(previous_state == dummy_state);
-	previous_state = get_gui_state();
-	set_gui_state(frontend_state_idle);
-	gui_cursor_set(ROBBER_CURSOR,
-		       check_move_robber_or_pirate,
+	if (!robber_busy) {
+		/* Do this only once.  */
+		robber_busy = TRUE;
+		g_assert(previous_state == dummy_state);
+		previous_state = get_gui_state();
+	}
+	/* These things are redone at undo.  */
+	set_gui_state(frontend_state_robber);
+	gui_cursor_set(ROBBER_CURSOR, check_move_robber_or_pirate,
 		       place_robber_or_pirate_cb, NULL, NULL);
 	gui_prompt_show(_("Place the robber"));
 	frontend_gui_update();
@@ -804,20 +822,16 @@ static void frontend_mode_setup(GuiEvent event)
 		frontend_gui_check(GUI_SETTLEMENT,
 				   setup_can_build_settlement());
 		frontend_gui_check(GUI_FINISH, setup_can_finish());
-		guimap_start_single_click_build(setup_can_build_road(),
-						check_road_setup,
-						build_road_cb,
-						setup_can_build_ship(),
-						check_ship_setup,
-						build_ship_cb,
-						setup_can_build_bridge(),
-						check_bridge_setup,
-						build_bridge_cb,
-						setup_can_build_settlement
-						(), check_settlement_setup,
-						build_settlement_cb, FALSE,
-						NULL, NULL, FALSE, NULL,
-						NULL, NULL);
+		guimap_single_click_set_functions(check_road_setup,
+						  build_road_cb,
+						  check_ship_setup,
+						  build_ship_cb,
+						  check_bridge_setup,
+						  build_bridge_cb,
+						  check_settlement_setup,
+						  build_settlement_cb,
+						  NULL, NULL, NULL,
+						  NULL, NULL, NULL, NULL);
 		break;
 	case GUI_UNDO:
 		/* The user has pressed the "Undo" button.  Send a
