@@ -102,7 +102,8 @@ static void frontend_offline_gui(GuiEvent event)
 		connectable = FALSE;
 		have_dlg = FALSE;
 		cb_connect(connect_get_server(), connect_get_port(),
-			   connect_get_name(), connect_get_viewer());
+			   connect_get_name(), connect_get_viewer(),
+			   connect_get_style());
 		frontend_gui_update();
 		break;
 	case GUI_CONNECT:
@@ -162,6 +163,7 @@ void frontend_init(int argc, char **argv)
 #endif
 	GOptionContext *context;
 	GError *error = NULL;
+	gchar *style;
 
 	frontend_gui_register_init();
 
@@ -238,8 +240,15 @@ void frontend_init(int argc, char **argv)
 							0) ? TRUE : FALSE;
 		}
 	}
+	style = config_get_string("connect/style", &default_returned);
+	if (default_returned) {
+		style = g_strdup("square");
+	}
+
 	connect_set_name(name);
 	connect_set_viewer(viewer);
+	connect_set_style(style);
+	g_free(style);
 
 	if (server && port) {
 		server_from_commandline = TRUE;
