@@ -21,11 +21,13 @@
  */
 
 #include "config.h"
+#include <stdlib.h>
 #include "colors.h"
 #include "frontend.h"
 #include "log.h"
 #include "gtkbugs.h"
 #include "common_gtk.h"
+#include "player-icon.h"
 
 static void player_show_connected_at_iter(gint player_num,
 					  gboolean connected,
@@ -33,6 +35,7 @@ static void player_show_connected_at_iter(gint player_num,
 
 static GdkColor ps_settlement = { 0, 0xbb00, 0x0000, 0x0000 };
 static GdkColor ps_city = { 0, 0xff00, 0x0000, 0x0000 };
+static GdkColor ps_city_wall = { 0, 0xff00, 0x0000, 0x0000 };
 static GdkColor ps_largest = { 0, 0x1c00, 0xb500, 0xed00 };
 static GdkColor ps_soldier = { 0, 0xe500, 0x8f00, 0x1600 };
 static GdkColor ps_resource = { 0, 0x0000, 0x0000, 0xFF00 };
@@ -48,6 +51,7 @@ typedef struct {
 static Statistic statistics[] = {
 	{N_("Settlement"), N_("Settlements"), &ps_settlement},
 	{N_("City"), N_("Cities"), &ps_city},
+	{N_("City Wall"), N_("City Walls"), &ps_city_wall},
 	{N_("Largest Army"), NULL, &ps_largest},
 	{N_("Longest Road"), NULL, &ps_largest},
 	{N_("Chapel"), N_("Chapels"), &ps_building},
@@ -99,15 +103,8 @@ static const gint turn_area_icon_width = 30;
 
 void player_init(void)
 {
-	gint idx;
-	GdkColormap *cmap;
-
 	colors_init();
-	cmap = gdk_colormap_get_system();
-	for (idx = 0; idx < MAX_PLAYERS; idx++) {
-		/* initialize their pixmap to 0 so it isn't unref'd */
-		players[idx].user_data = NULL;
-	}
+	playericon_init();
 }
 
 GdkColor *player_color(gint player_num)
@@ -127,48 +124,10 @@ GdkColor *player_or_viewer_color(gint player_num)
 GdkPixbuf *player_create_icon(GtkWidget * widget, gint player_num,
 			      gboolean connected)
 {
-	int width, height;
-	GdkPixbuf *temp;
-	GdkPixmap *pixmap;
-	GdkGC *gc;
-	Player *player;
-
-	gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &width, &height);
-
-	pixmap =
-	    gdk_pixmap_new(widget->window, width, height,
-			   gdk_visual_get_system()->depth);
-
-	gc = gdk_gc_new(pixmap);
-	gdk_gc_set_foreground(gc, &black);
-	gdk_draw_rectangle(pixmap, gc, TRUE, 0, 0, width, height);
-
-	gdk_gc_set_foreground(gc, player_color(player_num));
-	gdk_draw_rectangle(pixmap, gc, TRUE, 1, 1, width - 2, height - 2);
-
-	if (!connected) {
-		gdk_gc_set_foreground(gc, &black);
-		gdk_draw_rectangle(pixmap, gc, FALSE,
-				   3, 3, width - 7, height - 7);
-		gdk_draw_rectangle(pixmap, gc, FALSE,
-				   6, 6, width - 13, height - 13);
-	}
-
-	/* Store the pixmap in player->user_data. 
-	 * This is needed when the gtk_clist is still used 
-	 */
-	player = player_get(player_num);
-	if (player->user_data)
-		g_object_unref(player->user_data);
-	player->user_data = pixmap;
-	g_object_ref(pixmap);
-
-	temp = gdk_pixbuf_get_from_drawable(NULL, pixmap, NULL,
-					    0, 0, 0, 0, -1, -1);
-
-	g_object_unref(gc);
-	g_object_unref(pixmap);
-	return temp;
+	return playericon_create_icon(widget, player_get_style(player_num),
+				      player_or_viewer_color(player_num),
+				      player_is_viewer(player_num),
+				      connected, FALSE);
 }
 
 /** Locate a line suitable for the statistic */
@@ -480,6 +439,16 @@ void frontend_player_name(gint player_num, const gchar * name)
 	chat_player_name(player_num, name);
 }
 
+void frontend_player_style(gint player_num,
+			   G_GNUC_UNUSED const gchar * style)
+{
+	GtkTreeIter iter;
+
+	player_create_find_player(player_num, &iter);
+	player_show_connected_at_iter(player_num, TRUE, &iter);
+	chat_player_style(player_num);
+}
+
 void frontend_viewer_name(gint viewer_num, const gchar * name)
 {
 	GtkTreeIter iter;
