@@ -30,6 +30,7 @@
 
 #include "aboutbox.h"
 #include "game.h"
+#include "version.h"
 
 static GtkWidget *about = NULL;	/* The about window */
 
@@ -65,17 +66,28 @@ void aboutbox_display(const gchar * title, const gchar ** authors)
 			   FALSE, 0);
 
 	buffer = gtk_text_buffer_new(NULL);
-	gtk_text_buffer_get_start_iter(GTK_TEXT_BUFFER(buffer), &iter);
-	gtk_text_buffer_create_tag(GTK_TEXT_BUFFER(buffer), "bold",
+	gtk_text_buffer_get_start_iter(buffer, &iter);
+	gtk_text_buffer_create_tag(buffer, "bold",
 				   "weight", PANGO_WEIGHT_BOLD, NULL);
 
-	gtk_text_buffer_insert(GTK_TEXT_BUFFER(buffer), &iter,
+	gtk_text_buffer_insert(buffer, &iter,
 			       _("Pioneers is based upon the excellent\n"
 				 "Settlers of Catan board game.\n"), -1);
-	gtk_text_buffer_insert(buffer, &iter, VERSION, -1);
 	gtk_text_buffer_insert_with_tags_by_name(buffer, &iter,
-						 _("\nAuthors:\n"), -1,
+						 _("Version"), -1,
 						 "bold", NULL);
+	gtk_text_buffer_insert(buffer, &iter, ": ", -1);
+	gtk_text_buffer_insert(buffer, &iter, FULL_VERSION, -1);
+	gtk_text_buffer_insert(buffer, &iter, "\n", -1);
+	gtk_text_buffer_insert_with_tags_by_name(buffer, &iter,
+						 _("Homepage"), -1,
+						 "bold", NULL);
+	gtk_text_buffer_insert(buffer, &iter,
+			       ": http://pio.sourceforge.net\n", -1);
+	gtk_text_buffer_insert_with_tags_by_name(buffer, &iter,
+						 _("Authors"), -1, "bold",
+						 NULL);
+	gtk_text_buffer_insert(buffer, &iter, ":\n", -1);
 
 	for (i = 0; authors[i] != NULL; i++) {
 		if (i != 0)
@@ -84,17 +96,17 @@ void aboutbox_display(const gchar * title, const gchar ** authors)
 		gtk_text_buffer_insert(buffer, &iter, authors[i], -1);
 	}
 
-	gtk_text_buffer_get_start_iter(GTK_TEXT_BUFFER(buffer), &iter);
-	gtk_text_buffer_place_cursor(GTK_TEXT_BUFFER(buffer), &iter);
+	gtk_text_buffer_get_start_iter(buffer, &iter);
+	gtk_text_buffer_place_cursor(buffer, &iter);
 
-	view = gtk_text_view_new_with_buffer(GTK_TEXT_BUFFER(buffer));
+	view = gtk_text_view_new_with_buffer(buffer);
 	gtk_text_view_set_editable(GTK_TEXT_VIEW(view), FALSE);
 	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(about)->vbox), view, FALSE,
 			   FALSE, 0);
 
 	/* XXX GTK+ 2.6
 	   gtk_show_about_dialog(NULL,
-	   "version", VERSION,
+	   "version", FULL_VERSION,
 	   "copyright", _("(C) 2002 the Free Software Foundation"),
 	   "comments",
 	   _("Pioneers is based upon the excellent\n"
