@@ -69,23 +69,16 @@ on the surface.
 
 #include "config.h"
 #include <glib.h>
-#ifdef HAVE_GLIB_2_6
 #include <glib/gi18n.h>
+#include <glib/gstdio.h>
 #include <fcntl.h>
 #include <sys/stat.h>
 #include <unistd.h>
 #include <stdlib.h>
-#else				/* HAVE_GLIB_2_6 */
-#ifdef GNOME_DISABLE_DEPRECATED
-#undef GNOME_DISABLE_DEPRECATED
-#endif				/* GNOME_DISABLE_DEPRECATED */
-#include <libgnome/gnome-config.h>
-#endif				/* HAVE_GLIB_2_6 */
 #include "config-gnome.h"
 
 /* initialize configuration setup */
 
-#ifdef HAVE_GLIB_2_6
 static GKeyFile *keyfile = NULL;
 static gchar *filename = NULL;
 
@@ -102,41 +95,32 @@ static void config_sync(void)
 	if (!error) {
 		int f = open(filename, O_WRONLY | O_CREAT | O_TRUNC,
 			     S_IRUSR | S_IWUSR);
-#ifndef G_OS_WIN32
 		if (f == -1) {
 			/* Create the config dir, if it is missing */
 			/* Access mode: 0700 (drwx------) */
-			mkdir(g_get_user_config_dir(), S_IRWXU);
+			g_mkdir(g_get_user_config_dir(), S_IRWXU);
 			f = open(filename, O_WRONLY | O_CREAT | O_TRUNC,
 				 S_IRUSR | S_IWUSR);
 		};
-#endif
 		if (f != -1) {
 			write(f, data, length);
 			close(f);
 		} else {
-			g_warning(_("Could not write settings file"));
+			g_warning("Could not write settings file");
 		}
 	} else {
-		g_warning(_("Could not write settings file: %s"),
+		g_warning("Could not write settings file: %s",
 			  error->message);
 		g_error_free(error);
 	}
 }
 
-#endif				/* HAVE_GLIB_2_6 */
-
-/* set the prefix in some static manner so that we don't need to hard-code 
+/* set the prefix in some static manner so that we don't need to hard-code
  * it in the main code.  Thus, different architectures can have different 
  * prefixes depending on what's relevant for said arch.
  */
 void config_init(const gchar * path_prefix)
 {
-#ifndef HAVE_GLIB_2_6
-	gchar *path = g_strdup_printf("/%s/", path_prefix);
-	gnome_config_push_prefix(path);
-	g_free(path);
-#else				/* HAVE_GLIB_2_6 */
 	GError *error = NULL;
 
 	/* Don't initialize more than once */
@@ -152,19 +136,16 @@ void config_init(const gchar * path_prefix)
 				  G_KEY_FILE_KEEP_TRANSLATIONS, &error);
 
 	if (error != NULL) {
-		g_warning(_("Error while loading settings: %s"),
+		g_warning("Error while loading settings: %s",
 			  error->message);
 		g_error_free(error);
 	}
-#endif				/* HAVE_GLIB_2_6 */
 }
 
 void config_finish(void)
 {
-#ifdef HAVE_GLIB_2_6
 	g_free(filename);
 	g_key_file_free(keyfile);
-#endif				/* HAVE_GLIB_2_6 */
 }
 
 /* get configuration settings */
@@ -174,11 +155,6 @@ void config_finish(void)
  */
 gchar *config_get_string(const gchar * path, gboolean * default_used)
 {
-#ifndef HAVE_GLIB_2_6
-	/* gnome_config takes care of setting *default_used */
-	return gnome_config_get_string_with_default(path, default_used);
-#else				/* HAVE_GLIB_2_6 */
-
 	gchar **tokens;
 	gchar *value;
 	GError *error = NULL;
@@ -191,19 +167,17 @@ gchar *config_get_string(const gchar * path, gboolean * default_used)
 	    g_key_file_get_string(keyfile, tokens[0], tokens[1], &error);
 	if (error != NULL) {
 		if (tokens[2] == NULL) {
-			*default_used = FALSE;
 			value = g_strdup("");
 		} else {
-			*default_used = TRUE;
 			value = g_strdup(tokens[2]);
 		}
+		*default_used = TRUE;
 		g_error_free(error);
 	} else {
 		*default_used = FALSE;
 	}
 	g_strfreev(tokens);
 	return value;
-#endif				/* HAVE_GLIB_2_6 */
 }
 
 /* get an integer.  If a default is sent as part of the path, and the
@@ -211,11 +185,6 @@ gchar *config_get_string(const gchar * path, gboolean * default_used)
  */
 gint config_get_int(const gchar * path, gboolean * default_used)
 {
-#ifndef HAVE_GLIB_2_6
-	/* gnome_config takes care of setting *default_used */
-	return gnome_config_get_int_with_default(path, default_used);
-#else				/* HAVE_GLIB_2_6 */
-
 	gchar **tokens;
 	gint value;
 	GError *error = NULL;
@@ -228,31 +197,22 @@ gint config_get_int(const gchar * path, gboolean * default_used)
 	    g_key_file_get_integer(keyfile, tokens[0], tokens[1], &error);
 	if (error != NULL) {
 		if (tokens[2] == NULL) {
-			*default_used = FALSE;
 			value = 0;
 		} else {
-			*default_used = TRUE;
 			value = atoi(tokens[2]);
 		}
+		*default_used = TRUE;
 		g_error_free(error);
 	} else {
 		*default_used = FALSE;
 	}
 	g_strfreev(tokens);
 	return value;
-#endif				/* HAVE_GLIB_2_6 */
 }
 
 /* get an integer.  If the setting is not found, return the default value */
 gint config_get_int_with_default(const gchar * path, gint default_value)
 {
-#ifndef HAVE_GLIB_2_6
-	gchar temp[256];
-	gboolean default_used;
-
-	snprintf(temp, sizeof(temp), "%s=%d", path, default_value);
-	return gnome_config_get_int_with_default(temp, &default_used);
-#else				/* HAVE_GLIB_2_6 */
 	gboolean default_used;
 	gchar *temp;
 	gint value;
@@ -261,7 +221,6 @@ gint config_get_int_with_default(const gchar * path, gint default_value)
 	value = config_get_int(temp, &default_used);
 	g_free(temp);
 	return value;
-#endif				/* HAVE_GLIB_2_6 */
 }
 
 /* set configuration settings */
@@ -270,11 +229,6 @@ gint config_get_int_with_default(const gchar * path, gint default_value)
 /* set a string; make sure the configuration set is sync'd afterwards. */
 void config_set_string(const gchar * path, const gchar * value)
 {
-#ifndef HAVE_GLIB_2_6
-	gnome_config_set_string(path, value);
-	gnome_config_sync();
-#else				/* HAVE_GLIB_2_6 */
-
 	gchar **tokens;
 
 	g_return_if_fail(keyfile != NULL);
@@ -282,24 +236,18 @@ void config_set_string(const gchar * path, const gchar * value)
 	tokens = g_strsplit_set(path, "/", 2);
 
 	if (tokens[1] == NULL) {
-		g_warning(_("Key is missing"));
+		g_warning("Key is missing");
 	} else {
 		g_key_file_set_string(keyfile, tokens[0], tokens[1],
 				      value);
 	}
 	g_strfreev(tokens);
 	config_sync();
-#endif				/* HAVE_GLIB_2_6 */
 }
 
 /* set an int; make sure the configuration set is sync'd afterwards. */
 void config_set_int(const gchar * path, gint value)
 {
-#ifndef HAVE_GLIB_2_6
-	gnome_config_set_int(path, value);
-	gnome_config_sync();
-#else				/* HAVE_GLIB_2_6 */
-
 	gchar **tokens;
 
 	g_return_if_fail(keyfile != NULL);
@@ -307,12 +255,11 @@ void config_set_int(const gchar * path, gint value)
 	tokens = g_strsplit_set(path, "/", 2);
 
 	if (tokens[1] == NULL) {
-		g_warning(_("Key is missing"));
+		g_warning("Key is missing");
 	} else {
 		g_key_file_set_integer(keyfile, tokens[0], tokens[1],
 				       value);
 	}
 	g_strfreev(tokens);
 	config_sync();
-#endif				/* HAVE_GLIB_2_6 */
 }
