@@ -3,7 +3,7 @@
  *
  * Copyright (C) 1999 Dave Cole
  * Copyright (C) 2003 Bas Wijnen <shevek@fmf.nl>
- * Copyright (C) 2004 Roland Clobus <rclobus@bigfoot.com>
+ * Copyright (C) 2004-2007 Roland Clobus <rclobus@bigfoot.com>
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -54,6 +54,8 @@ static void calc_node_poly(const GuiMap * gmap, const Node * node,
 static void calc_hex_poly(const GuiMap * gmap, const Hex * hex,
 			  const Polygon * shape, Polygon * poly,
 			  double scale_factor, gint x_shift);
+static void guimap_cursor_move(GuiMap * gmap, gint x, gint y,
+			       MapElement * element);
 
 /* Square */
 static gint sqr(gint a)
@@ -77,6 +79,57 @@ GuiMap *guimap_new(void)
 	return gmap;
 }
 
+void guimap_delete(GuiMap * gmap)
+{
+	gint idx;
+
+	if (gmap->area != NULL) {
+		g_object_unref(gmap->area);
+		gmap->area = NULL;
+	}
+	if (gmap->pixmap != NULL) {
+		g_object_unref(gmap->pixmap);
+		gmap->pixmap = NULL;
+	}
+	if (gmap->gc != NULL) {
+		g_object_unref(gmap->gc);
+		gmap->gc = NULL;
+	}
+	if (gmap->hex_region != NULL) {
+		gdk_region_destroy(gmap->hex_region);
+		gmap->hex_region = NULL;
+	}
+	for (idx = 0; idx < 6; idx++) {
+		if (gmap->edge_region[idx] != NULL) {
+			gdk_region_destroy(gmap->edge_region[idx]);
+			gmap->edge_region[idx] = NULL;
+		}
+		if (gmap->node_region[idx] != NULL) {
+			gdk_region_destroy(gmap->node_region[idx]);
+			gmap->node_region[idx] = NULL;
+		}
+	}
+	if (gmap->layout) {
+		/* Restore the font size */
+		PangoContext *pc;
+		PangoFontDescription *pfd;
+
+		pc = pango_layout_get_context(gmap->layout);
+		pfd = pango_context_get_font_description(pc);
+
+		if (gmap->initial_font_size != -1) {
+			pango_font_description_set_size(pfd,
+							gmap->
+							initial_font_size);
+		}
+
+		g_object_unref(gmap->layout);
+		gmap->layout = NULL;
+	}
+	gmap->map = NULL;
+	g_free(gmap);
+}
+
 void guimap_reset(GuiMap * gmap)
 {
 	gmap->highlight_chit = -1;
@@ -158,6 +211,7 @@ static gint motion_notify_map_cb(GtkWidget * area, GdkEventMotion * event,
 GtkWidget *guimap_build_drawingarea(GuiMap * gmap, gint width, gint height)
 {
 	gmap->area = gtk_drawing_area_new();
+	g_object_ref(gmap->area);
 
 	gtk_widget_set_events(gmap->area, GDK_EXPOSURE_MASK
 			      | GDK_POINTER_MOTION_MASK
@@ -187,12 +241,19 @@ static Polygon settlement_poly = {
 };
 static GdkPoint city_points[] = {
 	{40, 20}, {40, -16}, {2, -16}, {2, -28},
-	{-18, -48}, {-40, -28}, {-40, 20}, {40, 20}
+	{-19, -48}, {-40, -28}, {-40, 20}, {40, 20}
 };
 static Polygon city_poly = {
 	city_points,
 	G_N_ELEMENTS(city_points)
 };
+static GdkPoint city_wall_points[] = {
+	{50, 36}, {50, -64}, {-50, -64}, {-50, 36}
+};
+static Polygon city_wall_poly = {
+	city_wall_points,
+	G_N_ELEMENTS(city_wall_points)
+};
 static GdkPoint nosetup_points[] = {
 	{0, 30}, {26, 15}, {26, -15}, {0, -30},
 	{-26, -15}, {-26, 15}, {0, 30}
@@ -201,6 +262,23 @@ static Polygon nosetup_poly = {
 	nosetup_points,
 	G_N_ELEMENTS(nosetup_points)
 };
+
+/* Update this when a node polygon is changed */
+#define NODE_MIN_X -50
+#define NODE_MAX_X 50
+#define NODE_MIN_Y -64
+#define NODE_MAX_Y 36
+
+static GdkPoint largest_node_points[] = {
+	{NODE_MIN_X, NODE_MIN_Y},
+	{NODE_MIN_X, NODE_MAX_Y},
+	{NODE_MAX_X, NODE_MAX_Y},
+	{NODE_MAX_X, NODE_MIN_Y}
+};
+static Polygon largest_node_poly = {
+	largest_node_points,
+	G_N_ELEMENTS(largest_node_points)
+};
 static GdkPoint road_points[] = {
 	{10, 40}, {10, -40}, {-10, -40}, {-10, 40},
 	{10, 40}
@@ -228,6 +306,24 @@ static Polygon bridge_poly = {
 	bridge_points,
 	G_N_ELEMENTS(bridge_points)
 };
+
+/* Update this when an edge polygon is changed */
+#define EDGE_MIN_X -14
+#define EDGE_MAX_X 50
+#define EDGE_MIN_Y -40
+#define EDGE_MAX_Y 40
+
+static GdkPoint largest_edge_points[] = {
+	{EDGE_MIN_X, EDGE_MIN_Y},
+	{EDGE_MIN_X, EDGE_MAX_Y},
+	{EDGE_MAX_X, EDGE_MAX_Y},
+	{EDGE_MAX_X, EDGE_MIN_Y}
+};
+static Polygon largest_edge_poly = {
+	largest_edge_points,
+	G_N_ELEMENTS(largest_edge_points)
+};
+
 static GdkPoint robber_points[] = {
 	{30, 60}, {30, 4}, {28, -6}, {22, -15},
 	{12, -20}, {22, -32}, {22, -48}, {10, -60},
@@ -441,6 +537,12 @@ void guimap_settlement_polygon(const GuiMap * gmap, const Node * node,
 	calc_node_poly(gmap, node, &settlement_poly, poly);
 }
 
+void guimap_city_wall_polygon(const GuiMap * gmap, const Node * node,
+			      Polygon * poly)
+{
+	calc_node_poly(gmap, node, &city_wall_poly, poly);
+}
+
 static void guimap_nosetup_polygon(const GuiMap * gmap, const Node * node,
 				   Polygon * poly)
 {
@@ -562,7 +664,7 @@ static gboolean display_hex(const Map * map, const Hex * hex,
 	}
 
 	/* Draw the dice roll */
-	if (hex->roll > 0) {
+	if (hex->roll > 0 && gmap->chit_radius > 0) {
 		g_assert(gmap->layout);
 		draw_dice_roll(gmap->layout, gmap->pixmap, gmap->gc,
 			       x_offset, y_offset, gmap->chit_radius,
@@ -572,7 +674,7 @@ static gboolean display_hex(const Map * map, const Hex * hex,
 	}
 
 	/* Draw ports */
-	if (hex->resource != NO_RESOURCE) {
+	if (hex->resource != NO_RESOURCE && gmap->chit_radius > 0) {
 		const gchar *str = "";
 		gint width, height;
 		int tileno = hex->resource == ANY_RESOURCE ?
@@ -713,40 +815,49 @@ static gboolean display_hex(const Map * map, const Hex * hex,
 			g_assert_not_reached();
 			break;
 		}
-		gdk_gc_set_foreground(gmap->gc,
-				      colors_get_player(edge->owner));
-		poly_draw(gmap->pixmap, gmap->gc, TRUE, &poly);
-		gdk_gc_set_foreground(gmap->gc, &black);
-		poly_draw(gmap->pixmap, gmap->gc, FALSE, &poly);
+		poly_draw_with_border(gmap->pixmap, gmap->gc,
+				      colors_get_player(edge->owner),
+				      &black, &poly);
 	}
 
 	/* Draw all buildings */
 	for (idx = 0; idx < G_N_ELEMENTS(hex->nodes); idx++) {
 		const Node *node = hex->nodes[idx];
+		const GdkColor *color;
+
 		if (node->owner < 0 && !gmap->show_nosetup_nodes)
 			continue;
 
-		poly.num_points = G_N_ELEMENTS(points);
 		if (node->owner < 0) {
 			if (!node->no_setup)
 				continue;
+			color = &white;
+			poly.num_points = G_N_ELEMENTS(points);
 			guimap_nosetup_polygon(gmap, node, &poly);
-			gdk_gc_set_foreground(gmap->gc, &white);
 		} else {
+			color = colors_get_player(node->owner);
+
+			/* If there are city walls, draw them. */
+			if (node->city_wall) {
+				poly.num_points = G_N_ELEMENTS(points);
+				guimap_city_wall_polygon(gmap, node,
+							 &poly);
+				poly_draw_with_border(gmap->pixmap,
+						      gmap->gc, color,
+						      &black, &poly);
+			}
 			/* Draw the building */
+			color = colors_get_player(node->owner);
+			poly.num_points = G_N_ELEMENTS(points);
 			if (node->type == BUILD_CITY)
 				guimap_city_polygon(gmap, node, &poly);
 			else
 				guimap_settlement_polygon(gmap, node,
 							  &poly);
-			gdk_gc_set_foreground(gmap->gc,
-					      colors_get_player(node->
-								owner));
 		}
 
-		poly_draw(gmap->pixmap, gmap->gc, TRUE, &poly);
-		gdk_gc_set_foreground(gmap->gc, &black);
-		poly_draw(gmap->pixmap, gmap->gc, FALSE, &poly);
+		poly_draw_with_border(gmap->pixmap, gmap->gc, color,
+				      &black, &poly);
 	}
 
 	/* Draw the robber */
@@ -796,27 +907,22 @@ void guimap_scale_with_radius(GuiMap * gmap, gint radius)
 {
 	int idx;
 
+	if (radius < MIN_HEX_RADIUS)
+		radius = MIN_HEX_RADIUS;
+
 	gmap->hex_radius = radius;
 	gmap->x_point = radius * cos(M_PI / 6.0);
 	gmap->y_point = radius * sin(M_PI / 6.0);
 
-/* Unused calculations ...
-	gint edge_width = radius / 10;
-	gint edge_len = radius / 5;
-	gint edge_x = edge_len * cos(M_PI / 6.0);
-	gint edge_y = edge_len * sin(M_PI / 6.0);
-	gint edge_x_point = edge_width * cos(M_PI / 3.0);
-	gint edge_y_point = edge_width * sin(M_PI / 3.0);
-*/
-	gmap->x_margin = gmap->y_margin = 3;
+	gmap->x_margin = gmap->y_margin = 0;
 
 	if (gmap->map == NULL)
 		return;
 
-	gmap->width = 2 * gmap->x_margin
-	    + gmap->map->x_size * 2 * gmap->x_point + gmap->x_point;
-	gmap->height = 2 * gmap->y_margin
-	    + (gmap->map->y_size - 1) * (gmap->hex_radius + gmap->y_point)
+	gmap->width =
+	    gmap->map->x_size * 2 * gmap->x_point + gmap->x_point;
+	gmap->height =
+	    (gmap->map->y_size - 1) * (gmap->hex_radius + gmap->y_point)
 	    + 2 * gmap->hex_radius;
 
 	if (gmap->map->shrink_left)
@@ -850,14 +956,15 @@ void guimap_scale_with_radius(GuiMap * gmap, gint radius)
 
 void guimap_scale_to_size(GuiMap * gmap, gint width, gint height)
 {
+	const gint reserved_width = 0;
+	const gint reserved_height = 0;
 	gint width_radius;
 	gint height_radius;
-
-	width_radius = (width - 20)
+	width_radius = (width - reserved_width)
 	    / ((gmap->map->x_size * 2 + 1
 		- gmap->map->shrink_left
 		- gmap->map->shrink_right) * cos(M_PI / 6.0));
-	height_radius = (height - 20)
+	height_radius = (height - reserved_height)
 	    / ((gmap->map->y_size - 1)
 	       * (sin(M_PI / 6.0) + 1) + 2);
 
@@ -961,7 +1068,7 @@ void guimap_display(GuiMap * gmap)
 	 */
 	maximum_size = gmap->hex_radius * 2 / 3;
 
-	/* First try to fix the text and the dots in the chit */
+	/* First attempt to fit the text and the dots in the chit */
 	pango_font_description_set_size(pfd, font_size);
 	pango_layout_set_font_description(gmap->layout, pfd);
 
@@ -976,8 +1083,11 @@ void guimap_display(GuiMap * gmap)
 		size_for_text =
 		    guimap_get_chit_radius(gmap->layout, FALSE);
 	};
-
-	gmap->chit_radius = size_for_text;
+	if (font_size <= 0) {
+		gmap->chit_radius = 0;
+	} else {
+		gmap->chit_radius = size_for_text;
+	}
 
 	map_traverse(gmap->map, (HexFunc) display_hex, gmap);
 }
@@ -1191,13 +1301,20 @@ static void find_hex(GuiMap * gmap, gint x, gint y, MapElement * element)
 	element->hex = find_hex_internal(gmap, x, y);
 }
 
-static void redraw_edge(GuiMap * gmap, const Edge * edge,
-			const Polygon * poly)
+void guimap_draw_edge(GuiMap * gmap, const Edge * edge)
 {
 	GdkRectangle rect;
 	gint idx;
+	Polygon poly;
+	GdkPoint points[MAX_POINTS];
 
-	poly_bound_rect(poly, 1, &rect);
+	g_return_if_fail(edge != NULL);
+	g_return_if_fail(gmap->pixmap != NULL);
+
+	poly.num_points = G_N_ELEMENTS(points);
+	poly.points = points;
+	calc_edge_poly(gmap, edge, &largest_edge_poly, &poly);
+	poly_bound_rect(&poly, 1, &rect);
 	gdk_gc_set_fill(gmap->gc, GDK_TILED);
 	gdk_gc_set_tile(gmap->gc,
 			theme_get_current()->terrain_tiles[BOARD_TILE]);
@@ -1208,7 +1325,7 @@ static void redraw_edge(GuiMap * gmap, const Edge * edge,
 		if (edge->hexes[idx] != NULL)
 			display_hex(gmap->map, edge->hexes[idx], gmap);
 
-	gdk_window_invalidate_rect(gmap->area->window, &rect, FALSE);	/* @todo FALSE or TRUE ? */
+	gdk_window_invalidate_rect(gmap->area->window, &rect, FALSE);
 }
 
 static void draw_cursor(GuiMap * gmap, gint owner, const Polygon * poly)
@@ -1220,71 +1337,17 @@ static void draw_cursor(GuiMap * gmap, gint owner, const Polygon * poly)
 	gdk_gc_set_line_attributes(gmap->gc, 2, GDK_LINE_SOLID,
 				   GDK_CAP_BUTT, GDK_JOIN_MITER);
 	gdk_gc_set_fill(gmap->gc, GDK_SOLID);
-	gdk_gc_set_foreground(gmap->gc, colors_get_player(owner));
-	poly_draw(gmap->pixmap, gmap->gc, TRUE, poly);
-
-	gdk_gc_set_foreground(gmap->gc, &green);
-	poly_draw(gmap->pixmap, gmap->gc, FALSE, poly);
+	poly_draw_with_border(gmap->pixmap, gmap->gc,
+			      colors_get_player(owner), &green, poly);
 
 	poly_bound_rect(poly, 1, &rect);
-	gdk_window_invalidate_rect(gmap->area->window, &rect, FALSE);	/* @todo FALSE or TRUE */
+	gdk_window_invalidate_rect(gmap->area->window, &rect, FALSE);
 }
 
-static void redraw_road(GuiMap * gmap, const Edge * edge)
-{
-	GdkPoint points[MAX_POINTS];
-	Polygon poly;
-
-	g_return_if_fail(edge != NULL);
-
-	poly.points = points;
-	poly.num_points = G_N_ELEMENTS(points);
-	guimap_road_polygon(gmap, edge, &poly);
-	redraw_edge(gmap, edge, &poly);
-}
-
-static void redraw_ship(GuiMap * gmap, const Edge * edge)
-{
-	GdkPoint points[MAX_POINTS];
-	Polygon poly;
-
-	g_return_if_fail(edge != NULL);
-
-	poly.points = points;
-	poly.num_points = G_N_ELEMENTS(points);
-	guimap_ship_polygon(gmap, edge, &poly);
-	redraw_edge(gmap, edge, &poly);
-}
-
-static void redraw_bridge(GuiMap * gmap, const Edge * edge)
-{
-	GdkPoint points[MAX_POINTS];
-	Polygon poly;
-
-	g_return_if_fail(edge != NULL);
-
-	poly.points = points;
-	poly.num_points = G_N_ELEMENTS(points);
-	guimap_bridge_polygon(gmap, edge, &poly);
-	redraw_edge(gmap, edge, &poly);
-}
-
-static void erase_road_cursor(GuiMap * gmap)
+static void erase_edge_cursor(GuiMap * gmap)
 {
 	g_return_if_fail(gmap->cursor.pointer != NULL);
-	redraw_road(gmap, gmap->cursor.edge);
-}
-
-static void erase_ship_cursor(GuiMap * gmap)
-{
-	g_return_if_fail(gmap->cursor.pointer != NULL);
-	redraw_ship(gmap, gmap->cursor.edge);
-}
-
-static void erase_bridge_cursor(GuiMap * gmap)
-{
-	g_return_if_fail(gmap->cursor.pointer != NULL);
-	redraw_bridge(gmap, gmap->cursor.edge);
+	guimap_draw_edge(gmap, gmap->cursor.edge);
 }
 
 static void draw_road_cursor(GuiMap * gmap)
@@ -1326,25 +1389,25 @@ static void draw_bridge_cursor(GuiMap * gmap)
 	draw_cursor(gmap, gmap->cursor_owner, &poly);
 }
 
-static void redraw_node(GuiMap * gmap, const Node * node,
-			const Polygon * poly)
+void guimap_draw_node(GuiMap * gmap, const Node * node)
 {
 	GdkRectangle rect;
 	int idx;
+	Polygon poly;
+	GdkPoint points[MAX_POINTS];
 
 	g_return_if_fail(node != NULL);
 	g_return_if_fail(gmap->pixmap != NULL);
-	/* don't do anything if pixmap is not ready */
-	if (gmap->pixmap == NULL)
-		return;
 
-	poly_bound_rect(poly, 1, &rect);
+	poly.num_points = G_N_ELEMENTS(points);
+	poly.points = points;
+	calc_node_poly(gmap, node, &largest_node_poly, &poly);
+	poly_bound_rect(&poly, 1, &rect);
 	gdk_gc_set_fill(gmap->gc, GDK_TILED);
 	gdk_gc_set_tile(gmap->gc,
 			theme_get_current()->terrain_tiles[BOARD_TILE]);
 	gdk_draw_rectangle(gmap->pixmap, gmap->gc, TRUE,
 			   rect.x, rect.y, rect.width, rect.height);
-
 	for (idx = 0; idx < G_N_ELEMENTS(node->hexes); idx++)
 		if (node->hexes[idx] != NULL)
 			display_hex(gmap->map, node->hexes[idx], gmap);
@@ -1352,17 +1415,10 @@ static void redraw_node(GuiMap * gmap, const Node * node,
 	gdk_window_invalidate_rect(gmap->area->window, &rect, FALSE);
 }
 
-static void erase_settlement_cursor(GuiMap * gmap)
+static void erase_node_cursor(GuiMap * gmap)
 {
-	GdkPoint points[MAX_POINTS];
-	Polygon poly;
-
 	g_return_if_fail(gmap->cursor.pointer != NULL);
-
-	poly.points = points;
-	poly.num_points = G_N_ELEMENTS(points);
-	guimap_settlement_polygon(gmap, gmap->cursor.node, &poly);
-	redraw_node(gmap, gmap->cursor.node, &poly);
+	guimap_draw_node(gmap, gmap->cursor.node);
 }
 
 static void draw_settlement_cursor(GuiMap * gmap)
@@ -1378,19 +1434,6 @@ static void draw_settlement_cursor(GuiMap * gmap)
 	draw_cursor(gmap, gmap->cursor_owner, &poly);
 }
 
-static void erase_city_cursor(GuiMap * gmap)
-{
-	GdkPoint points[MAX_POINTS];
-	Polygon poly;
-
-	g_return_if_fail(gmap->cursor.pointer != NULL);
-
-	poly.points = points;
-	poly.num_points = G_N_ELEMENTS(points);
-	guimap_city_polygon(gmap, gmap->cursor.node, &poly);
-	redraw_node(gmap, gmap->cursor.node, &poly);
-}
-
 static void draw_city_cursor(GuiMap * gmap)
 {
 	GdkPoint points[MAX_POINTS];
@@ -1404,17 +1447,22 @@ static void draw_city_cursor(GuiMap * gmap)
 	draw_cursor(gmap, gmap->cursor_owner, &poly);
 }
 
-static void erase_steal_building_cursor(GuiMap * gmap)
+static void draw_city_wall_cursor(GuiMap * gmap)
 {
 	GdkPoint points[MAX_POINTS];
 	Polygon poly;
 
 	g_return_if_fail(gmap->cursor.pointer != NULL);
 
+	poly.points = points;
+	poly.num_points = G_N_ELEMENTS(points);
+	guimap_city_wall_polygon(gmap, gmap->cursor.node, &poly);
+	draw_cursor(gmap, gmap->cursor_owner, &poly);
+
 	poly.points = points;
 	poly.num_points = G_N_ELEMENTS(points);
 	guimap_city_polygon(gmap, gmap->cursor.node, &poly);
-	redraw_node(gmap, gmap->cursor.node, &poly);
+	draw_cursor(gmap, gmap->cursor_owner, &poly);
 }
 
 static void draw_steal_building_cursor(GuiMap * gmap)
@@ -1554,31 +1602,6 @@ void guimap_highlight_chits(GuiMap * gmap, gint roll)
 			     &closure);
 }
 
-void guimap_draw_edge(GuiMap * gmap, const Edge * edge)
-{
-	if (edge->type == BUILD_ROAD) {
-		redraw_road(gmap, edge);
-	} else if (edge->type == BUILD_SHIP) {
-		redraw_ship(gmap, edge);
-	} else if (edge->type == BUILD_BRIDGE) {
-		redraw_bridge(gmap, edge);
-	}
-}
-
-void guimap_draw_node(GuiMap * gmap, const Node * node)
-{
-	GdkPoint points[MAX_POINTS];
-	Polygon poly;
-
-	poly.points = points;
-	poly.num_points = G_N_ELEMENTS(points);
-	if (node->type == BUILD_SETTLEMENT)
-		guimap_settlement_polygon(gmap, node, &poly);
-	else
-		guimap_city_polygon(gmap, node, &poly);
-	redraw_node(gmap, node, &poly);
-}
-
 void guimap_draw_hex(GuiMap * gmap, const Hex * hex)
 {
 	GdkPoint points[MAX_POINTS];
@@ -1615,19 +1638,20 @@ typedef struct {
 /* This array must follow the enum CursorType */
 static ModeCursor cursors[] = {
 	{NULL, NULL, NULL},	/* NO_CURSOR */
-	{find_edge, erase_road_cursor, draw_road_cursor},	/* ROAD_CURSOR */
-	{find_edge, erase_ship_cursor, draw_ship_cursor},	/* SHIP_CURSOR */
-	{find_edge, erase_bridge_cursor, draw_bridge_cursor},	/* BRIDGE_CURSOR */
-	{find_node, erase_settlement_cursor, draw_settlement_cursor},	/* SETTLEMENT_CURSOR */
-	{find_node, erase_city_cursor, draw_city_cursor},	/* CITY_CURSOR */
-	{find_node, erase_steal_building_cursor, draw_steal_building_cursor},	/* STEAL_BUILDING_CURSOR */
-	{find_edge, erase_ship_cursor, draw_steal_ship_cursor},	/* STEAL_SHIP_CURSOR */
+	{find_edge, erase_edge_cursor, draw_road_cursor},	/* ROAD_CURSOR */
+	{find_edge, erase_edge_cursor, draw_ship_cursor},	/* SHIP_CURSOR */
+	{find_edge, erase_edge_cursor, draw_bridge_cursor},	/* BRIDGE_CURSOR */
+	{find_node, erase_node_cursor, draw_settlement_cursor},	/* SETTLEMENT_CURSOR */
+	{find_node, erase_node_cursor, draw_city_cursor},	/* CITY_CURSOR */
+	{find_node, erase_node_cursor, draw_city_wall_cursor},	/* CITY_WALL_CURSOR */
+	{find_node, erase_node_cursor, draw_steal_building_cursor},	/* STEAL_BUILDING_CURSOR */
+	{find_edge, erase_edge_cursor, draw_steal_ship_cursor},	/* STEAL_SHIP_CURSOR */
 	{find_hex, erase_robber_cursor, draw_robber_cursor}	/* ROBBER_CURSOR */
 };
 
-gboolean roadM, shipM, bridgeM, settlementM, cityM, shipMoveM;
-CheckFunc roadF, shipF, bridgeF, settlementF, cityF, shipMoveF;
-SelectFunc roadS, shipS, bridgeS, settlementS, cityS, shipMoveS;
+gboolean roadM, shipM, bridgeM, settlementM, cityM, cityWallM, shipMoveM;
+CheckFunc roadF, shipF, bridgeF, settlementF, cityF, cityWallF, shipMoveF;
+SelectFunc roadS, shipS, bridgeS, settlementS, cityS, cityWallS, shipMoveS;
 CancelFunc shipMoveC;
 
 /** Calculate the distance between the element and the cursor.
@@ -1677,6 +1701,7 @@ void guimap_cursor_move(GuiMap * gmap, gint x, gint y,
 		gboolean can_build_bridge = FALSE;
 		gboolean can_build_settlement = FALSE;
 		gboolean can_build_city = FALSE;
+		gboolean can_build_city_wall = FALSE;
 		gboolean can_move_ship = FALSE;
 		gboolean can_build_edge = FALSE;
 		gboolean can_build_node = FALSE;
@@ -1776,8 +1801,13 @@ void guimap_cursor_move(GuiMap * gmap, gint x, gint y,
 					  && cityF(*element,
 						   gmap->player_num,
 						   dummyElement));
+			can_build_city_wall = (cityWallM
+					       && cityWallF(*element,
+							    gmap->
+							    player_num,
+							    dummyElement));
 			can_build_node = can_build_settlement
-			    || can_build_city;
+			    || can_build_city || can_build_city_wall;
 			if (can_build_node)
 				distance_node =
 				    distance_cursor(gmap, element,
@@ -1816,6 +1846,10 @@ void guimap_cursor_move(GuiMap * gmap, gint x, gint y,
 			guimap_cursor_set(gmap, CITY_CURSOR,
 					  gmap->player_num, cityF, cityS,
 					  NULL, NULL, TRUE);
+		else if (can_build_city_wall && can_build_node)
+			guimap_cursor_set(gmap, CITY_WALL_CURSOR,
+					  gmap->player_num, cityWallF,
+					  cityWallS, NULL, NULL, TRUE);
 		else if (can_move_ship && can_build_edge)
 			guimap_cursor_set(gmap, SHIP_CURSOR,
 					  gmap->player_num, shipMoveF,
@@ -1914,48 +1948,79 @@ void guimap_cursor_set(GuiMap * gmap, CursorType cursor_type, gint owner,
 	gmap->cursor.pointer = NULL;
 }
 
-void guimap_start_single_click_build(gboolean road_mask,
-				     CheckFunc road_check_func,
-				     SelectFunc road_select_func,
-				     gboolean ship_mask,
-				     CheckFunc ship_check_func,
-				     SelectFunc ship_select_func,
-				     gboolean bridge_mask,
-				     CheckFunc bridge_check_func,
-				     SelectFunc bridge_select_func,
-				     gboolean settlement_mask,
-				     CheckFunc settlement_check_func,
-				     SelectFunc settlement_select_func,
-				     gboolean city_mask,
-				     CheckFunc city_check_func,
-				     SelectFunc city_select_func,
-				     gboolean ship_move_mask,
-				     CheckFunc ship_move_check_func,
-				     SelectFunc ship_move_select_func,
-				     CancelFunc ship_move_cancel_func)
+void guimap_single_click_set_functions(CheckFunc road_check_func,
+				       SelectFunc road_select_func,
+				       CheckFunc ship_check_func,
+				       SelectFunc ship_select_func,
+				       CheckFunc bridge_check_func,
+				       SelectFunc bridge_select_func,
+				       CheckFunc
+				       settlement_check_func,
+				       SelectFunc
+				       settlement_select_func,
+				       CheckFunc city_check_func,
+				       SelectFunc city_select_func,
+				       CheckFunc city_wall_check_func,
+				       SelectFunc city_wall_select_func,
+				       CheckFunc
+				       ship_move_check_func,
+				       SelectFunc
+				       ship_move_select_func,
+				       CancelFunc ship_move_cancel_func)
 {
-	roadM = road_mask;
 	roadF = road_check_func;
 	roadS = road_select_func;
-	shipM = ship_mask;
 	shipF = ship_check_func;
 	shipS = ship_select_func;
-	bridgeM = bridge_mask;
 	bridgeF = bridge_check_func;
 	bridgeS = bridge_select_func;
-	settlementM = settlement_mask;
 	settlementF = settlement_check_func;
 	settlementS = settlement_select_func;
-	cityM = city_mask;
 	cityF = city_check_func;
 	cityS = city_select_func;
-	shipMoveM = ship_move_mask;
+	cityWallF = city_wall_check_func;
+	cityWallS = city_wall_select_func;
 	shipMoveF = ship_move_check_func;
 	shipMoveS = ship_move_select_func;
 	shipMoveC = ship_move_cancel_func;
 	single_click_build_active = TRUE;
 }
 
+void guimap_single_click_set_road_mask(gboolean mask)
+{
+	roadM = mask;
+}
+
+void guimap_single_click_set_ship_mask(gboolean mask)
+{
+	shipM = mask;
+}
+
+void guimap_single_click_set_bridge_mask(gboolean mask)
+{
+	bridgeM = mask;
+}
+
+void guimap_single_click_set_settlement_mask(gboolean mask)
+{
+	settlementM = mask;
+}
+
+void guimap_single_click_set_city_mask(gboolean mask)
+{
+	cityM = mask;
+}
+
+void guimap_single_click_set_city_wall_mask(gboolean mask)
+{
+	cityWallM = mask;
+}
+
+void guimap_single_click_set_ship_move_mask(gboolean mask)
+{
+	shipMoveM = mask;
+}
+
 void guimap_set_show_no_setup_nodes(GuiMap * gmap, gboolean show)
 {
 	gboolean old_show = gmap->show_nosetup_nodes;
