@@ -37,6 +37,42 @@ Hex *map_hex(Map * map, gint x, gint y)
 	return map->grid[y][x];
 }
 
+Hex *hex_in_direction(Map * map, const Hex * hex, HexDirection direction)
+{
+	gint x = hex->x;
+	gint y = hex->y;
+
+	switch (direction) {
+	case HEX_DIR_E:
+		x++;
+		break;
+	case HEX_DIR_NE:
+		if (y % 2 == 1)
+			x++;
+		y--;
+		break;
+	case HEX_DIR_NW:
+		if (y % 2 == 0)
+			x--;
+		y--;
+		break;
+	case HEX_DIR_W:
+		x--;
+		break;
+	case HEX_DIR_SW:
+		if (y % 2 == 0)
+			x--;
+		y++;
+		break;
+	case HEX_DIR_SE:
+		if (y % 2 == 1)
+			x++;
+		y++;
+		break;
+	}
+	return map_hex(map, x, y);
+}
+
 Node *map_node(Map * map, gint x, gint y, gint pos)
 {
 	Hex *hex;
@@ -385,6 +421,14 @@ void map_shuffle_terrain(Map * map)
 	gint num_terrain;
 	gint num_port;
 
+	/* Remove robber, because the desert will probably move.
+	 * It will be restored by layout_chits.
+	 */
+	if (map->robber_hex) {
+		map->robber_hex->robber = FALSE;
+		map->robber_hex = NULL;
+	}
+
 	/* Count number of each terrain type
 	 */
 	memset(terrain_count, 0, sizeof(terrain_count));
