@@ -64,10 +64,11 @@ typedef enum {
 	BUILD_SHIP,		/* ship was built */
 	BUILD_SETTLEMENT,	/* settlement was built */
 	BUILD_CITY,		/* city was built */
+	BUILD_CITY_WALL,	/* city wall was built */
 	BUILD_MOVE_SHIP		/* a ship was moved (only used for undo list) */
 } BuildType;
 
-#define NUM_BUILD_TYPES (BUILD_CITY + 1)
+#define NUM_BUILD_TYPES (BUILD_CITY_WALL + 1)
 
 /* Maps are built up from a network of hexes, edges, and nodes.
  *
@@ -109,6 +110,7 @@ struct _Node {
 
 	gboolean visited;	/* used for longest road */
 	gboolean no_setup;	/* setup is not allowed on this node */
+	gboolean city_wall;	/* has city wall */
 };
 
 struct _Edge {
@@ -153,9 +155,19 @@ typedef struct {
 	gboolean specific_resource[NO_RESOURCE];
 } MaritimeInfo;
 
+typedef enum {
+	HEX_DIR_E,
+	HEX_DIR_NE,
+	HEX_DIR_NW,
+	HEX_DIR_W,
+	HEX_DIR_SW,
+	HEX_DIR_SE
+} HexDirection;
+
 /* map.c
  */
 Hex *map_hex(Map * map, gint x, gint y);
+Hex *hex_in_direction(Map * map, const Hex * hex, HexDirection direction);
 Edge *map_edge(Map * map, gint x, gint y, gint pos);
 Node *map_node(Map * map, gint x, gint y, gint pos);
 typedef gboolean(*HexFunc) (Map * map, Hex * hex, void *closure);
@@ -199,12 +211,14 @@ gboolean can_settlement_be_setup(const Node * node);
 gboolean can_settlement_be_built(const Node * node, gint owner);
 gboolean can_settlement_be_upgraded(const Node * node, gint owner);
 gboolean can_city_be_built(const Node * node, int owner);
+gboolean can_city_wall_be_built(const Node * node, int owner);
 gboolean can_robber_or_pirate_be_moved(const Hex * hex);
 /* map global queries */
 gboolean map_can_place_road(Map * map, int owner);
 gboolean map_can_place_ship(Map * map, int owner);
 gboolean map_can_place_bridge(Map * map, int owner);
 gboolean map_can_place_settlement(Map * map, int owner);
+gboolean map_can_place_city_wall(Map * map, int owner);
 gboolean map_can_upgrade_settlement(Map * map, int owner);
 
 gboolean map_building_spacing_ok(Map * map, gint owner, BuildType type,
@@ -226,6 +240,7 @@ gboolean map_bridge_connect_ok(Map * map, gint owner, gint x, gint y,
 void map_longest_road(Map * map, gint * lengths, gint num_players);
 gboolean map_is_island_discovered(Map * map, Node * node, gint owner);
 void map_maritime_info(Map * map, MaritimeInfo * info, gint owner);
+guint map_count_islands(Map * map);
 
 extern GRand *g_rand_ctx;
 
