@@ -3,7 +3,8 @@
  *
  * Copyright (C) 1999 Dave Cole
  * Copyright (C) 2003, 2006 Bas Wijnen <shevek@fmf.nl>
- * Copyright (C) 2005,2006 Roland Clobus <rclobus@bigfoot.com>
+ * Copyright (C) 2005-2007 Roland Clobus <rclobus@bigfoot.com>
+ * Copyright (C) 2005 Keishi Suenaga
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -152,7 +153,39 @@ static void notify(Session * ses, NetEvent event, gchar * line)
 		ses->notify_func(event, ses->user_data, line);
 }
 
-void net_close(Session * ses)
+static gboolean net_would_block(void)
+{
+#ifdef G_OS_WIN32
+	return WSAGetLastError() == WSAEWOULDBLOCK;
+#else				/* G_OS_WIN32 */
+	return errno == EAGAIN;
+#endif				/* G_OS_WIN32 */
+}
+
+static gboolean net_write_error(void)
+{
+#ifdef G_OS_WIN32
+	int lerror = WSAGetLastError();
+	return (lerror != WSAECONNRESET
+		&& lerror != WSAECONNABORTED && lerror != WSAESHUTDOWN);
+#else				/* G_OS_WIN32 */
+	return errno != EPIPE;
+#endif				/* G_OS_WIN32 */
+}
+
+/* Returns the message for error# number */
+static const gchar *net_errormsg_nr(gint number)
+{
+	return g_strerror(number);
+}
+
+/* Returns the message for the current error */
+static const gchar *net_errormsg(void)
+{
+	return net_errormsg_nr(errno);
+}
+
+gboolean net_close(Session * ses)
 {
 	if (ses->timer_id != 0) {
 		g_source_remove(ses->timer_id);
@@ -173,6 +206,7 @@ void net_close(Session * ses)
 			g_free(data);
 		}
 	}
+	return !ses->entered;
 }
 
 void net_close_when_flushed(Session * ses)
@@ -181,8 +215,8 @@ void net_close_when_flushed(Session * ses)
 	if (ses->write_queue != NULL)
 		return;
 
-	net_close(ses);
-	notify(ses, NET_CLOSE, NULL);
+	if (net_close(ses))
+		notify(ses, NET_CLOSE, NULL);
 }
 
 void net_wait_for_close(Session * ses)
@@ -192,8 +226,8 @@ void net_wait_for_close(Session * ses)
 
 static void close_and_callback(Session * ses)
 {
-	net_close(ses);
-	notify(ses, NET_CLOSE, NULL);
+	if (net_close(ses))
+		notify(ses, NET_CLOSE, NULL);
 }
 
 static gboolean ping_function(gpointer s)
@@ -206,7 +240,8 @@ static gboolean ping_function(gpointer s)
 		/* There was no response to the ping in time.  The connection
 		 * should be considered dead.  */
 		log_message(MSG_ERROR,
-			    "No activity and no response to ping.  Closing connection");
+			    "No activity and no response to ping.  Closing connection\n");
+		debug("(%d) --> %s", ses->fd, "no response");
 		close_and_callback(ses);
 	} else if (interval >= PING_PERIOD) {
 		/* There was no activity.
@@ -244,14 +279,14 @@ static void write_ready(Session * ses)
 			log_message(MSG_ERROR,
 				    _
 				    ("Error checking connect status: %s\n"),
-				    g_strerror(errno));
+				    net_errormsg());
 			net_close(ses);
 		} else if (error != 0) {
 			notify(ses, NET_CONNECT_FAIL, NULL);
 			log_message(MSG_ERROR,
 				    _
 				    ("Error connecting to host '%s': %s\n"),
-				    ses->host, g_strerror(error));
+				    ses->host, net_errormsg_nr(error));
 			net_close(ses);
 		} else {
 			ses->connect_in_progress = FALSE;
@@ -271,13 +306,13 @@ static void write_ready(Session * ses)
 		debug("write_ready: write(%d, \"%.*s\", %d) = %d",
 		      ses->fd, len, data, len, num);
 		if (num < 0) {
-			if (errno == EAGAIN)
+			if (net_would_block())
 				break;
-			if (errno != EPIPE)
+			if (net_write_error())
 				log_message(MSG_ERROR,
 					    _
 					    ("Error writing socket: %s\n"),
-					    g_strerror(errno));
+					    net_errormsg());
 			close_and_callback(ses);
 			return;
 		} else if (num == len) {
@@ -316,17 +351,19 @@ void net_write(Session * ses, const gchar * data)
 
 		len = strlen(data);
 		num = send(ses->fd, data, len, 0);
-		if (num > 0)
-			debug("(%d) --> %s", ses->fd, data);
-		else if (errno != EAGAIN)
+		if (num > 0) {
+			if (strcmp(data, "yes\n")
+			    && strcmp(data, "hello\n"))
+				debug("(%d) --> %s", ses->fd, data);
+		} else if (!net_would_block())
 			debug("(%d) --- Error writing to socket.",
 			      ses->fd);
 		if (num < 0) {
-			if (errno != EAGAIN) {
+			if (!net_would_block()) {
 				log_message(MSG_ERROR,
 					    _
 					    ("Error writing to socket: %s\n"),
-					    g_strerror(errno));
+					    net_errormsg());
 				close_and_callback(ses);
 				return;
 			}
@@ -386,10 +423,10 @@ static void read_ready(Session * ses)
 	num = recv(ses->fd, ses->read_buff + ses->read_len,
 		   sizeof(ses->read_buff) - ses->read_len, 0);
 	if (num < 0) {
-		if (errno == EAGAIN)
+		if (net_would_block())
 			return;
 		log_message(MSG_ERROR, _("Error reading socket: %s\n"),
-			    g_strerror(errno));
+			    net_errormsg());
 		close_and_callback(ses);
 		return;
 	}
@@ -415,8 +452,6 @@ static void read_ready(Session * ses)
 		line[len] = '\0';
 		offset += len + 1;
 
-		debug("(%d) <-- %s", ses->fd, line);
-
 		if (!strcmp(line, "hello")) {
 			net_write(ses, "yes\n");
 			continue;
@@ -425,6 +460,8 @@ static void read_ready(Session * ses)
 			continue;	/* Don't notify the program */
 		}
 
+		debug("(%d) <-- %s", ses->fd, line);
+
 		notify(ses, NET_READ, line);
 	}
 
@@ -442,6 +479,9 @@ static void read_ready(Session * ses)
 		ses->read_len = 0;
 
 	ses->entered = FALSE;
+	if (ses->fd < 0) {
+		close_and_callback(ses);
+	}
 }
 
 Session *net_new(NetNotifyFunc notify_func, void *user_data)
@@ -555,7 +595,7 @@ gboolean net_connect(Session * ses, const gchar * host, const gchar * port)
 		if (ses->fd < 0) {
 			log_message(MSG_ERROR,
 				    _("Error creating socket: %s\n"),
-				    g_strerror(errno));
+				    net_errormsg());
 			continue;
 		}
 #ifdef HAVE_FCNTL
@@ -563,7 +603,7 @@ gboolean net_connect(Session * ses, const gchar * host, const gchar * port)
 			log_message(MSG_ERROR,
 				    _
 				    ("Error setting socket close-on-exec: %s\n"),
-				    g_strerror(errno));
+				    net_errormsg());
 			net_closesocket(ses->fd);
 			ses->fd = -1;
 			continue;
@@ -573,7 +613,7 @@ gboolean net_connect(Session * ses, const gchar * host, const gchar * port)
 			log_message(MSG_ERROR,
 				    _
 				    ("Error setting socket non-blocking: %s\n"),
-				    g_strerror(errno));
+				    net_errormsg());
 			net_closesocket(ses->fd);
 			ses->fd = -1;
 			continue;
@@ -598,7 +638,7 @@ gboolean net_connect(Session * ses, const gchar * host, const gchar * port)
 				log_message(MSG_ERROR,
 					    _
 					    ("Error connecting to %s: %s\n"),
-					    host, g_strerror(errno));
+					    host, net_errormsg());
 				net_closesocket(ses->fd);
 				ses->fd = -1;
 				continue;
@@ -617,11 +657,13 @@ gboolean net_connect(Session * ses, const gchar * host, const gchar * port)
 		return FALSE;
 }
 
-/* Free and NULL-ity the session *ses */
+/* Free and NULL-ify the session *ses */
 void net_free(Session ** ses)
 {
-	net_close(*ses);
-
+	/* If the sessions is still in use, do not free it */
+	if (!net_close(*ses)) {
+		return;
+	}
 	if ((*ses)->host != NULL)
 		g_free((*ses)->host);
 	if ((*ses)->port != NULL)
@@ -721,18 +763,18 @@ int net_open_listening_socket(const gchar * port, gchar ** error_message)
 		*error_message =
 		    g_strdup_printf(_
 				    ("Error creating listening socket: %s\n"),
-				    g_strerror(errno));
+				    net_errormsg());
 		freeaddrinfo(ai);
 		return -1;
 	}
 
 	freeaddrinfo(ai);
 
-	if (fcntl(fd, F_SETFL, O_NDELAY) < 0) {
+	if (net_set_socket_non_blocking(fd)) {
 		*error_message =
 		    g_strdup_printf(_
 				    ("Error setting socket non-blocking: %s\n"),
-				    g_strerror(errno));
+				    net_errormsg());
 		net_closesocket(fd);
 		return -1;
 	}
@@ -741,7 +783,7 @@ int net_open_listening_socket(const gchar * port, gchar ** error_message)
 		*error_message =
 		    g_strdup_printf(_
 				    ("Error during listen on socket: %s\n"),
-				    g_strerror(errno));
+				    net_errormsg());
 		net_closesocket(fd);
 		return -1;
 	}
@@ -779,7 +821,7 @@ gboolean net_get_peer_name(gint fd, gchar ** hostname, gchar ** servname,
 	if (getpeername(fd, &peer.sa, &peer_len) < 0) {
 		*error_message =
 		    g_strdup_printf(_("Error getting peer name: %s"),
-				    g_strerror(errno));
+				    net_errormsg());
 		return FALSE;
 	} else {
 		int err;
@@ -821,7 +863,7 @@ gint net_accept(gint accept_fd, gchar ** error_message)
 	if (fd < 0) {
 		*error_message =
 		    g_strdup_printf(_("Error accepting connection: %s"),
-				    g_strerror(errno));
+				    net_errormsg());
 	}
 	return fd;
 }
