@@ -110,7 +110,7 @@ static void route_event(StateMachine * sm, gint event)
 			sm->unhandled(user_data, event);
 		break;
 	case SM_NET_CLOSE:
-		net_free(&(sm->ses));
+		sm_close(sm);
 	default:
 		curr_state(user_data, event);
 		if (!sm->is_dead && sm->global !=NULL)
@@ -260,6 +260,10 @@ static gint try_recv(StateMachine * sm, const gchar * fmt, va_list ap)
 				   == 0) {
 				*build_type = BUILD_SETTLEMENT;
 				offset += 10;
+			} else if (strncmp(line + offset, "city_wall", 9)
+				   == 0) {
+				*build_type = BUILD_CITY_WALL;
+				offset += 9;
 			} else if (strncmp(line + offset, "city", 4) == 0) {
 				*build_type = BUILD_CITY;
 				offset += 4;
@@ -386,6 +390,9 @@ gchar *sm_vformat(const gchar * fmt, va_list ap)
 			case BUILD_CITY:
 				buff_append(result, "%s", "city");
 				break;
+			case BUILD_CITY_WALL:
+				buff_append(result, "%s", "city_wall");
+				break;
 			case BUILD_NONE:
 				g_error("BUILD_NONE passed to sm_vformat");
 				break;
@@ -420,7 +427,7 @@ void sm_write(StateMachine * sm, const gchar * str)
 	if (sm->use_cache) {
 		/* Protect against strange/slow connects */
 		if (g_list_length(sm->cache) > 1000) {
-			net_write(sm->ses, "ERR connection too slow");
+			net_write(sm->ses, "ERR connection too slow\n");
 			net_close_when_flushed(sm->ses);
 		} else {
 			sm->cache =
@@ -430,6 +437,14 @@ void sm_write(StateMachine * sm, const gchar * str)
 		net_write(sm->ses, str);
 }
 
+void sm_write_uncached(StateMachine * sm, const gchar * str)
+{
+	g_assert(sm->ses);
+	g_assert(sm->use_cache);
+
+	net_write(sm->ses, str);
+}
+
 void sm_send(StateMachine * sm, const gchar * fmt, ...)
 {
 	va_list ap;
@@ -468,32 +483,6 @@ void sm_set_use_cache(StateMachine * sm, gboolean use_cache)
 	sm->use_cache = use_cache;
 }
 
-StateMachine *sm_copy_as_uncached(const StateMachine * sm)
-{
-	StateMachine *copy;
-	copy = g_malloc(sizeof(*sm));
-	memcpy(copy, sm, sizeof(*sm));
-	copy->use_cache = FALSE;
-	copy->cache = NULL;
-	return copy;
-}
-
-void sm_send_uncached(StateMachine * sm, const gchar * fmt, ...)
-{
-	va_list ap;
-	gchar *buff;
-
-	g_assert(sm->ses);
-	g_assert(sm->use_cache);
-
-	va_start(ap, fmt);
-	buff = sm_vformat(fmt, ap);
-	va_end(ap);
-
-	net_write(sm->ses, buff);
-	g_free(buff);
-}
-
 void sm_global_set(StateMachine * sm, StateFunc state)
 {
 	sm->global = state;
@@ -661,6 +650,7 @@ void sm_free(StateMachine * sm)
 {
 	if (sm->ses != NULL) {
 		net_free(&(sm->ses));
+		g_return_if_fail(sm->ses == NULL);
 	}
 	if (sm->use_count > 0)
 		sm->is_dead = TRUE;
@@ -673,6 +663,17 @@ void sm_free(StateMachine * sm)
 void sm_close(StateMachine * sm)
 {
 	net_free(&(sm->ses));
+	if (sm->use_cache) {
+		/* Purge the cache */
+		GList *list = sm->cache;
+		sm->cache = NULL;
+		sm_set_use_cache(sm, FALSE);
+		while (list) {
+			gchar *data = list->data;
+			list = g_list_remove(list, data);
+			g_free(data);
+		}
+	}
 }
 
 void sm_stack_dump(StateMachine * sm)
