@@ -21,6 +21,9 @@
  */
 
 #include "config.h"
+#ifdef HAVE_LOCALE_H
+#include <locale.h>
+#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
