@@ -3,7 +3,7 @@
  *
  * Copyright (C) 1999 Dave Cole
  * Copyright (C) 2003 Bas Wijnen <shevek@fmf.nl>
- * Copyright (C) 2004-2006 Roland Clobus <rclobus@bigfoot.com>
+ * Copyright (C) 2004-2007 Roland Clobus <rclobus@bigfoot.com>
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,6 +22,10 @@
 
 #include "config.h"
 #include "version.h"
+
+#ifdef HAVE_LOCALE_H
+#include <locale.h>
+#endif
 #include <ctype.h>
 #include <gtk/gtk.h>
 #include <string.h>
@@ -36,13 +40,17 @@
 
 #include "select-game.h"	/* Custom widget */
 #include "game-settings.h"	/* Custom widget */
+#include "game-rules.h"
+#include "theme.h"
 
 #define MAINICON_FILE	"pioneers-server.png"
 
 static GtkWidget *settings_notebook;	/* relevant settings */
 static GtkWidget *game_frame;	/* the frame containing all settings regarding the game */
+static GtkWidget *rules_frame;	/* the frame containing all rules regarding the game */
 static GtkWidget *select_game;	/* select game type */
 static GtkWidget *game_settings;	/* the settings of the game */
+static GtkWidget *game_rules;	/* the rules of the game */
 static GtkWidget *server_frame;	/* the frame containing all settings regarding the server */
 static GtkWidget *ai_frame;	/* the frame containing all settings regarding the ai */
 static GtkWidget *register_toggle;	/* register with meta server? */
@@ -52,6 +60,7 @@ static GtkWidget *overridden_hostname_entry;	/* name of server (allows masquerad
 static GtkWidget *port_entry;	/* server port */
 static GtkWidget *random_toggle;	/* randomize seating order? */
 static GtkWidget *addcomputer_btn;	/* button to add computer players */
+static GtkWidget *launchclient_btn;	/* button to launch client window */
 
 static GtkWidget *start_btn;	/* start/stop the server */
 static GtkTooltips *tooltips;	/* tooltips */
@@ -71,6 +80,7 @@ static gboolean show_version = FALSE;
 
 /* Local function prototypes */
 static void add_game_to_list(gpointer name, gpointer user_data);
+static void check_vp_cb(GObject * caller, gpointer main_window);
 static void quit_cb(void);
 static void help_about_cb(void);
 
@@ -87,6 +97,9 @@ enum {
 static GtkActionEntry entries[] = {
 	{"GameMenu", NULL, N_("_Game"), NULL, NULL, NULL},
 	{"HelpMenu", NULL, N_("_Help"), NULL, NULL, NULL},
+	{"GameCheckVP", GTK_STOCK_APPLY, N_("_Check Victory Point Target"),
+	 NULL,
+	 N_("Check whether the game can be won"), G_CALLBACK(check_vp_cb)},
 	{"GameQuit", GTK_STOCK_QUIT, N_("_Quit"), "<control>Q",
 	 N_("Quit the program"), quit_cb},
 	{"HelpAbout", NULL, N_("_About Pioneers Server"), NULL,
@@ -98,6 +111,8 @@ static const char *ui_description =
 "<ui>"
 "  <menubar name='MainMenu'>"
 "    <menu action='GameMenu'>"
+"      <menuitem action='GameCheckVP' />"
+"      <separator/>"
 "      <menuitem action='GameQuit' />"
 "    </menu>"
 "    <menu action='HelpMenu'>"
@@ -121,11 +136,7 @@ static void port_entry_changed_cb(GtkWidget * widget,
 static void register_toggle_cb(GtkToggleButton * toggle,
 			       G_GNUC_UNUSED gpointer user_data)
 {
-	GtkWidget *label = GTK_BIN(toggle)->child;
-
 	register_server = gtk_toggle_button_get_active(toggle);
-	gtk_label_set_text(GTK_LABEL(label),
-			   register_server ? _("Yes") : _("No"));
 	gtk_widget_set_sensitive(meta_entry, register_server);
 	gtk_widget_set_sensitive(overridden_hostname_entry,
 				 register_server);
@@ -134,21 +145,13 @@ static void register_toggle_cb(GtkToggleButton * toggle,
 static void random_toggle_cb(GtkToggleButton * toggle,
 			     G_GNUC_UNUSED gpointer user_data)
 {
-	GtkWidget *label = GTK_BIN(toggle)->child;
-
 	random_order = gtk_toggle_button_get_active(toggle);
-	gtk_label_set_text(GTK_LABEL(label),
-			   random_order ? _("Yes") : _("No"));
 }
 
 static void chat_toggle_cb(GtkToggleButton * toggle,
 			   G_GNUC_UNUSED gpointer user_data)
 {
-	GtkWidget *label = GTK_BIN(toggle)->child;
-
 	want_ai_chat = gtk_toggle_button_get_active(toggle);
-	gtk_label_set_text(GTK_LABEL(label),
-			   want_ai_chat ? _("Yes") : _("No"));
 }
 
 /* The server does not need to respond to changed game settings directly
@@ -169,14 +172,21 @@ static void update_game_settings(const GameParams * params)
 	g_return_if_fail(params != NULL);
 
 	/* Update the UI */
-	game_settings_set_terrain(GAMESETTINGS(game_settings),
-				  params->random_terrain);
 	game_settings_set_players(GAMESETTINGS(game_settings),
 				  params->num_players);
 	game_settings_set_victory_points(GAMESETTINGS(game_settings),
 					 params->victory_points);
-	game_settings_set_sevens_rule(GAMESETTINGS(game_settings),
-				      params->sevens_rule);
+	game_rules_set_random_terrain(GAMERULES(game_rules),
+				      params->random_terrain);
+	game_rules_set_sevens_rule(GAMERULES(game_rules),
+				   params->sevens_rule);
+	game_rules_set_domestic_trade(GAMERULES(game_rules),
+				      params->domestic_trade);
+	game_rules_set_strict_trade(GAMERULES(game_rules),
+				    params->strict_trade);
+	game_rules_set_use_pirate(GAMERULES(game_rules),
+				  params->use_pirate,
+				  params->num_build_type[BUILD_SHIP]);
 }
 
 static void game_activate(GtkWidget * widget,
@@ -236,9 +246,6 @@ static void start_clicked_cb(G_GNUC_UNUSED GtkButton * start_btn,
 
 		title = select_game_get_active(SELECTGAME(select_game));
 		params = params_copy(game_list_find_item(title));
-		cfg_set_terrain_type(params,
-				     game_settings_get_terrain(GAMESETTINGS
-							       (game_settings)));
 		cfg_set_num_players(params,
 				    game_settings_get_players(GAMESETTINGS
 							      (game_settings)));
@@ -246,9 +253,19 @@ static void start_clicked_cb(G_GNUC_UNUSED GtkButton * start_btn,
 				       game_settings_get_victory_points
 				       (GAMESETTINGS(game_settings)));
 		cfg_set_sevens_rule(params,
-				    game_settings_get_sevens_rule
-				    (GAMESETTINGS(game_settings)));
+				    game_rules_get_sevens_rule
+				    (GAMERULES(game_rules)));
+		cfg_set_terrain_type(params,
+				     game_rules_get_random_terrain
+				     (GAMERULES(game_rules)));
+		params->strict_trade =
+		    game_rules_get_strict_trade(GAMERULES(game_rules));
+		params->use_pirate =
+		    game_rules_get_use_pirate(GAMERULES(game_rules));
+		params->domestic_trade =
+		    game_rules_get_domestic_trade(GAMERULES(game_rules));
 		update_game_settings(params);
+
 		g_assert(server_port != NULL);
 		if (start_server
 		    (params, overridden_hostname, server_port,
@@ -272,11 +289,53 @@ static void start_clicked_cb(G_GNUC_UNUSED GtkButton * start_btn,
 				       params->victory_points);
 			config_set_int("game/sevens-rule",
 				       params->sevens_rule);
+			config_set_int("game/use-pirate",
+				       params->use_pirate);
+			config_set_int("game/strict-trade",
+				       params->strict_trade);
+			config_set_int("game/domestic-trade",
+				       params->domestic_trade);
 		}
 		params_free(params);
 	}
 }
 
+static void launchclient_clicked_cb(G_GNUC_UNUSED GtkButton * start_btn,
+				    G_GNUC_UNUSED gpointer user_data)
+{
+	gchar *child_argv[7];
+	GSpawnFlags child_flags = G_SPAWN_STDOUT_TO_DEV_NULL |
+	    G_SPAWN_STDERR_TO_DEV_NULL;
+	GError *error = NULL;
+	gint i;
+
+	g_assert(server_port != NULL);
+
+	/* Populate argv. */
+	child_argv[0] = g_strdup(PIONEERS_CLIENT_GTK_PATH);
+	child_argv[1] = g_strdup(PIONEERS_CLIENT_GTK_PATH);
+	child_argv[2] = g_strdup("-s");
+	child_argv[3] = g_strdup("localhost");
+	child_argv[4] = g_strdup("-p");
+	child_argv[5] = g_strdup(server_port);
+	child_argv[6] = NULL;
+
+	/* Spawn the child. */
+	if (!g_spawn_async(NULL, child_argv, NULL, child_flags,
+			   NULL, NULL, NULL, &error)) {
+		/* Error message when program %1 is started, reason is %2 */
+		log_message(MSG_ERROR,
+			    _("Error starting %s: %s"),
+			    PIONEERS_CLIENT_GTK_PATH, error->message);
+		g_error_free(error);
+	}
+
+	/* Clean up after ourselves. */
+	for (i = 0; child_argv[i] != NULL; i++) {
+		g_free(child_argv[i]);
+	}
+}
+
 static void addcomputer_clicked_cb(G_GNUC_UNUSED GtkButton * start_btn,
 				   G_GNUC_UNUSED gpointer user_data)
 {
@@ -351,7 +410,8 @@ static void add_game_to_list(gpointer name,
 			     G_GNUC_UNUSED gpointer user_data)
 {
 	GameParams *a = (GameParams *) name;
-	select_game_add(SELECTGAME(select_game), a->title);
+	select_game_add_with_map(SELECTGAME(select_game), a->title,
+				 a->map);
 }
 
 static void overridden_hostname_changed_cb(GtkEntry * widget,
@@ -379,14 +439,11 @@ static void meta_server_changed_cb(GtkWidget * widget,
 	meta_server_name = text;
 }
 
-static GtkWidget *build_game_settings(GtkWidget * parent)
+static GtkWidget *build_game_settings(GtkWidget * parent,
+				      GtkWindow * main_window)
 {
 	GtkWidget *frame;
 	GtkWidget *vbox;
-	gboolean default_returned;
-	gchar *gamename;
-	gint temp;
-	GameParams *params;
 
 	tooltips = gtk_tooltips_new();
 
@@ -405,14 +462,102 @@ static GtkWidget *build_game_settings(GtkWidget * parent)
 			 G_CALLBACK(game_activate), NULL);
 	gtk_box_pack_start(GTK_BOX(vbox), select_game, FALSE, FALSE, 0);
 
-	game_settings = game_settings_new();
+	game_settings = game_settings_new(TRUE);
 	gtk_widget_show(game_settings);
 	gtk_box_pack_start(GTK_BOX(vbox), game_settings, TRUE, TRUE, 0);
-	/* The server does not need to respond directly to each change.
-	 * The parameters will be read when the start button is clicked
-	 g_signal_connect(G_OBJECT(game_settings), "change",
-	 G_CALLBACK(game_settings_change_cb), NULL);
-	 */
+
+	g_signal_connect(G_OBJECT(game_settings), "check",
+			 G_CALLBACK(check_vp_cb), main_window);
+
+	return frame;
+}
+
+static GtkWidget *build_game_rules(GtkWidget * parent)
+{
+	GtkWidget *frame;
+	GtkWidget *vbox;
+
+	tooltips = gtk_tooltips_new();
+
+	frame = gtk_frame_new(_("Rules"));
+	gtk_widget_show(frame);
+	gtk_box_pack_start(GTK_BOX(parent), frame, FALSE, TRUE, 0);
+
+	vbox = gtk_vbox_new(FALSE, 3);
+	gtk_widget_show(vbox);
+	gtk_container_add(GTK_CONTAINER(frame), vbox);
+	gtk_container_set_border_width(GTK_CONTAINER(vbox), 3);
+
+	game_rules = game_rules_new();
+	gtk_widget_show(game_rules);
+	gtk_box_pack_start(GTK_BOX(vbox), game_rules, TRUE, TRUE, 0);
+
+	return frame;
+}
+
+static void
+my_cell_player_viewer_to_text(G_GNUC_UNUSED GtkTreeViewColumn *
+			      tree_column, GtkCellRenderer * cell,
+			      GtkTreeModel * tree_model,
+			      GtkTreeIter * iter, gpointer data)
+{
+	gboolean b;
+
+	/* Get the value from the model. */
+	gtk_tree_model_get(tree_model, iter, GPOINTER_TO_INT(data), &b,
+			   -1);
+	g_object_set(cell, "text", b ?
+		     /* Role of the player: viewer */
+		     _("Viewer") :
+		     /* Role of the player: player */
+		     _("Player"), NULL);
+}
+
+static GtkWidget *build_interface(GtkWindow * main_window)
+{
+	GtkWidget *vbox;
+	GtkWidget *vbox_settings;
+	GtkWidget *vbox_connected;
+	GtkWidget *vbox_ai;
+	GtkWidget *frame;
+	GtkWidget *table;
+	GtkWidget *label;
+	GtkWidget *scroll_win;
+	GtkWidget *message_text;
+	GtkTooltips *tooltips;
+	gint novar;
+	gboolean default_returned;
+	gchar *gamename;
+	gint temp;
+	GameParams *params;
+
+	GtkCellRenderer *renderer;
+	GtkTreeViewColumn *column;
+
+	tooltips = gtk_tooltips_new();
+
+	vbox = gtk_vbox_new(FALSE, 5);
+	gtk_widget_show(vbox);
+	gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
+
+	settings_notebook = gtk_notebook_new();
+	gtk_widget_show(settings_notebook);
+	gtk_notebook_set_show_border(GTK_NOTEBOOK(settings_notebook),
+				     FALSE);
+	gtk_box_pack_start(GTK_BOX(vbox), settings_notebook, FALSE, TRUE,
+			   0);
+
+	vbox_settings = gtk_vbox_new(FALSE, 5);
+	gtk_widget_show(vbox_settings);
+	gtk_notebook_append_page(GTK_NOTEBOOK(settings_notebook),
+				 vbox_settings,
+				 gtk_label_new(_("Game Settings")));
+
+	/* Game settings frame */
+	game_frame = build_game_settings(vbox_settings, main_window);
+
+	/* Rules frame */
+	rules_frame = build_game_rules(vbox_settings);
 
 	/* Fill the GUI with the saved settings */
 	gamename =
@@ -440,68 +585,17 @@ static GtkWidget *build_game_settings(GtkWidget * parent)
 	temp = config_get_int("game/sevens-rule", &default_returned);
 	if (!default_returned)
 		cfg_set_sevens_rule(params, temp);
-
+	temp = config_get_int("game/use-pirate", &default_returned);
+	if (!default_returned)
+		params->use_pirate = temp;
+	temp = config_get_int("game/strict-trade", &default_returned);
+	if (!default_returned)
+		params->strict_trade = temp;
+	temp = config_get_int("game/domestic-trade", &default_returned);
+	if (!default_returned)
+		params->domestic_trade = temp;
 	update_game_settings(params);
 	params_free(params);
-	return frame;
-}
-
-static void
-my_cell_player_viewer_to_text(G_GNUC_UNUSED GtkTreeViewColumn *
-			      tree_column, GtkCellRenderer * cell,
-			      GtkTreeModel * tree_model,
-			      GtkTreeIter * iter, gpointer data)
-{
-	gboolean b;
-
-	/* Get the value from the model. */
-	gtk_tree_model_get(tree_model, iter, GPOINTER_TO_INT(data), &b,
-			   -1);
-	g_object_set(cell, "text", b ?
-		     /* Role of the player: viewer */
-		     _("Viewer") :
-		     /* Role of the player: player */
-		     _("Player"), NULL);
-}
-
-static GtkWidget *build_interface(void)
-{
-	GtkWidget *vbox;
-	GtkWidget *vbox_settings;
-	GtkWidget *vbox_ai;
-	GtkWidget *hbox_ai;
-	GtkWidget *frame;
-	GtkWidget *table;
-	GtkWidget *label;
-	GtkWidget *scroll_win;
-	GtkWidget *message_text;
-	GtkTooltips *tooltips;
-	gint novar;
-
-	GtkCellRenderer *renderer;
-	GtkTreeViewColumn *column;
-
-	tooltips = gtk_tooltips_new();
-
-	vbox = gtk_vbox_new(FALSE, 5);
-	gtk_widget_show(vbox);
-	gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
-
-	settings_notebook = gtk_notebook_new();
-	gtk_widget_show(settings_notebook);
-	gtk_notebook_set_show_border(GTK_NOTEBOOK(settings_notebook),
-				     FALSE);
-	gtk_box_pack_start(GTK_BOX(vbox), settings_notebook, FALSE, TRUE,
-			   0);
-
-	vbox_settings = gtk_vbox_new(FALSE, 5);
-	gtk_widget_show(vbox_settings);
-	gtk_notebook_append_page(GTK_NOTEBOOK(settings_notebook),
-				 vbox_settings,
-				 gtk_label_new(_("Game Settings")));
-
-	/* Game settings frame */
-	game_frame = build_game_settings(vbox_settings);
 
 	server_frame = gtk_frame_new(_("Server Parameters"));
 	gtk_widget_show(server_frame);
@@ -526,20 +620,17 @@ static GtkWidget *build_interface(void)
 			 G_CALLBACK(port_entry_changed_cb), NULL);
 	gtk_widget_show(port_entry);
 	gtk_table_attach(GTK_TABLE(table), port_entry, 1, 2, 0, 1,
-			 GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+			 GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0,
+			 0);
 	gtk_tooltips_set_tip(tooltips, port_entry,
 			     _("The port for the game server"), NULL);
 
-	label = gtk_label_new(_("Register Server"));
-	gtk_widget_show(label);
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,
-			 GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
-	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-
-	register_toggle = gtk_toggle_button_new_with_label(_("No"));
+	register_toggle =
+	    gtk_check_button_new_with_label(_("Register Server"));
 	gtk_widget_show(register_toggle);
-	gtk_table_attach(GTK_TABLE(table), register_toggle, 1, 2, 1, 2,
-			 GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+	gtk_table_attach(GTK_TABLE(table), register_toggle, 0, 2, 1, 2,
+			 GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0,
+			 0);
 	g_signal_connect(G_OBJECT(register_toggle), "toggled",
 			 G_CALLBACK(register_toggle_cb), NULL);
 	gtk_tooltips_set_tip(tooltips, register_toggle,
@@ -557,7 +648,8 @@ static GtkWidget *build_interface(void)
 			 G_CALLBACK(meta_server_changed_cb), NULL);
 	gtk_widget_show(meta_entry);
 	gtk_table_attach(GTK_TABLE(table), meta_entry, 1, 2, 2, 3,
-			 GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+			 GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0,
+			 0);
 	gtk_tooltips_set_tip(tooltips, meta_entry,
 			     _("The address of the meta server"), NULL);
 
@@ -572,22 +664,19 @@ static GtkWidget *build_interface(void)
 			 G_CALLBACK(overridden_hostname_changed_cb), NULL);
 	gtk_widget_show(overridden_hostname_entry);
 	gtk_table_attach(GTK_TABLE(table), overridden_hostname_entry, 1, 2,
-			 3, 4, GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+			 3, 4, GTK_EXPAND | GTK_FILL,
+			 GTK_EXPAND | GTK_FILL, 0, 0);
 	gtk_tooltips_set_tip(tooltips, overridden_hostname_entry,
 			     _
 			     ("The public name of this computer (needed when playing behind a firewall)"),
 			     NULL);
 
-	label = gtk_label_new(_("Random Turn Order"));
-	gtk_widget_show(label);
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 4, 5,
-			 GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
-	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-
-	random_toggle = gtk_toggle_button_new_with_label(_("No"));
+	random_toggle =
+	    gtk_check_button_new_with_label(_("Random Turn Order"));
 	gtk_widget_show(random_toggle);
-	gtk_table_attach(GTK_TABLE(table), random_toggle, 1, 2, 4, 5,
-			 GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+	gtk_table_attach(GTK_TABLE(table), random_toggle, 0, 2, 4, 5,
+			 GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0,
+			 0);
 	g_signal_connect(G_OBJECT(random_toggle), "toggled",
 			 G_CALLBACK(random_toggle_cb), NULL);
 	gtk_tooltips_set_tip(tooltips, random_toggle,
@@ -636,9 +725,16 @@ static GtkWidget *build_interface(void)
 	gtk_widget_show(frame);
 	gtk_box_pack_start(GTK_BOX(vbox_settings), frame, TRUE, TRUE, 0);
 
+	vbox_connected = gtk_vbox_new(FALSE, 5);
+	gtk_widget_show(vbox_connected);
+	gtk_container_set_border_width(GTK_CONTAINER(vbox_connected), 5);
+	gtk_container_add(GTK_CONTAINER(frame), vbox_connected);
+
 	scroll_win = gtk_scrolled_window_new(NULL, NULL);
 	gtk_widget_show(scroll_win);
-	gtk_container_add(GTK_CONTAINER(frame), scroll_win);
+	gtk_box_pack_start(GTK_BOX(vbox_connected), scroll_win, TRUE, TRUE,
+			   0);
+
 	gtk_container_set_border_width(GTK_CONTAINER(scroll_win), 3);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll_win),
 				       GTK_POLICY_NEVER,
@@ -719,6 +815,16 @@ static GtkWidget *build_interface(void)
 
 	gtk_widget_show(label);
 
+	launchclient_btn =
+	    gtk_button_new_with_label(_("Launch Pioneers Client"));
+	gtk_widget_show(launchclient_btn);
+	gtk_box_pack_start(GTK_BOX(vbox_connected), launchclient_btn,
+			   FALSE, FALSE, 0);
+	g_signal_connect(G_OBJECT(launchclient_btn), "clicked",
+			 G_CALLBACK(launchclient_clicked_cb), NULL);
+	gtk_tooltips_set_tip(tooltips, launchclient_btn,
+			     _("Launch the Pioneers Client"), NULL);
+
 	ai_frame = gtk_frame_new(_("Computer Players"));
 	gtk_widget_show(ai_frame);
 	gtk_box_pack_start(GTK_BOX(vbox_settings), ai_frame, FALSE, FALSE,
@@ -727,18 +833,10 @@ static GtkWidget *build_interface(void)
 	gtk_widget_show(vbox_ai);
 	gtk_container_set_border_width(GTK_CONTAINER(vbox_ai), 5);
 	gtk_container_add(GTK_CONTAINER(ai_frame), vbox_ai);
-	hbox_ai = gtk_hbox_new(FALSE, 5);
-	gtk_widget_show(hbox_ai);
-	gtk_box_pack_start(GTK_BOX(vbox_ai), hbox_ai, FALSE, FALSE, 0);
 
-	label = gtk_label_new(_("Enable Chat"));
-	gtk_widget_show(label);
-	gtk_box_pack_start(GTK_BOX(hbox_ai), label, FALSE, FALSE, 0);
-	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-
-	chat_toggle = gtk_toggle_button_new_with_label(_("No"));
+	chat_toggle = gtk_check_button_new_with_label(_("Enable Chat"));
 	gtk_widget_show(chat_toggle);
-	gtk_box_pack_start(GTK_BOX(hbox_ai), chat_toggle, TRUE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox_ai), chat_toggle, TRUE, TRUE, 0);
 	g_signal_connect(G_OBJECT(chat_toggle), "toggled",
 			 G_CALLBACK(chat_toggle_cb), NULL);
 	gtk_tooltips_set_tip(tooltips, chat_toggle,
@@ -790,6 +888,37 @@ static GtkWidget *build_interface(void)
 	return vbox;
 }
 
+static void check_vp_cb(G_GNUC_UNUSED GObject * caller,
+			gpointer main_window)
+{
+	const gchar *title;
+	GameParams *params;
+
+	title = select_game_get_active(SELECTGAME(select_game));
+	params = params_copy(game_list_find_item(title));
+	cfg_set_num_players(params,
+			    game_settings_get_players(GAMESETTINGS
+						      (game_settings)));
+	cfg_set_victory_points(params,
+			       game_settings_get_victory_points
+			       (GAMESETTINGS(game_settings)));
+	cfg_set_sevens_rule(params,
+			    game_rules_get_sevens_rule
+			    (GAMERULES(game_rules)));
+	cfg_set_terrain_type(params,
+			     game_rules_get_random_terrain
+			     (GAMERULES(game_rules)));
+	params->strict_trade =
+	    game_rules_get_strict_trade(GAMERULES(game_rules));
+	params->use_pirate =
+	    game_rules_get_use_pirate(GAMERULES(game_rules));
+	params->domestic_trade =
+	    game_rules_get_domestic_trade(GAMERULES(game_rules));
+	update_game_settings(params);
+
+	check_victory_points(params, main_window);
+}
+
 static void quit_cb(void)
 {
 	gtk_main_quit();
@@ -811,6 +940,12 @@ void game_is_over(G_GNUC_UNUSED Game * game)
 	log_message(MSG_INFO, _("The game is over.\n"));
 }
 
+void request_server_stop(void)
+{
+	if (server_stop())
+		gui_set_server_state(FALSE);
+}
+
 static GOptionEntry commandline_entries[] = {
 	{"debug", '\0', 0, G_OPTION_ARG_NONE, &enable_debug,
 	 /* Commandline option of server-gtk: enable debug logging */
@@ -904,6 +1039,8 @@ int main(int argc, char *argv[])
 
 	config_init("pioneers-server");
 
+	themes_init();
+
 	icon_file =
 	    g_build_filename(DATADIR, "pixmaps", MAINICON_FILE, NULL);
 	if (g_file_test(icon_file, G_FILE_TEST_EXISTS)) {
@@ -916,7 +1053,8 @@ int main(int argc, char *argv[])
 	menubar = gtk_ui_manager_get_widget(ui_manager, "/MainMenu");
 	gtk_box_pack_start(GTK_BOX(vbox), menubar, FALSE, FALSE, 0);
 
-	gtk_box_pack_start(GTK_BOX(vbox), build_interface(), TRUE, TRUE,
+	gtk_box_pack_start(GTK_BOX(vbox),
+			   build_interface(GTK_WINDOW(window)), TRUE, TRUE,
 			   0);
 
 	gtk_widget_show_all(window);
