@@ -24,8 +24,6 @@
 #include <stdlib.h>
 #include "server.h"
 
-const gchar *meta_server_name = NULL;
-
 static Session *ses;
 static enum {
 	MODE_SIGNON,
@@ -48,7 +46,7 @@ gchar *get_server_name(void)
 	return server_name;
 }
 
-void meta_start_game()
+void meta_start_game(void)
 {
 #ifdef CLOSE_META_AT_START
 	if (ses != NULL) {
@@ -82,11 +80,10 @@ void meta_send_details(Game * game)
 		g_free(game->hostname);
 		game->hostname = NULL;
 	}
-	/* No hostname set, use default */
-	if (!game->hostname) {
-		game->hostname = get_server_name();
+	/* No hostname set, let the metaserver figure out our name */
+	if (game->hostname) {
+		net_printf(ses, "host=%s\n", game->hostname);
 	}
-	net_printf(ses, "host=%s\n", game->hostname);
 	if (meta_server_version_major >= 1) {
 		net_printf(ses,
 			   "vpoints=%d\n"
@@ -165,17 +162,16 @@ static void meta_event(NetEvent event, Game * game, char *line)
 			meta_mode = MODE_SERVER_LIST;
 			meta_send_details(game);
 			break;
-
-		case MODE_SERVER_LIST:
-			if (strcmp(line, "hello") == 0)
-				net_printf(ses, "yes\n");
+		default:
+			log_message(MSG_ERROR,
+				    _
+				    ("Unknown message from the metaserver: %s\n"),
+				    line);
 			break;
 		}
 		break;
 	case NET_CLOSE:
-		if (meta_mode == MODE_SIGNON)
-			log_message(MSG_ERROR,
-				    _("Meta-server kicked us off\n"));
+		log_message(MSG_ERROR, _("Meta-server kicked us off\n"));
 		net_free(&ses);
 		break;
 	case NET_CONNECT:
@@ -207,3 +203,11 @@ void meta_register(const gchar * server, const gchar * port, Game * game)
 		net_free(&ses);
 	}
 }
+
+void meta_unregister(void)
+{
+	if (ses != NULL) {
+		log_message(MSG_INFO, _("Unregister from meta-server\n"));
+		net_free(&ses);
+	}
+}
