@@ -2,7 +2,7 @@
  *   Go buy a copy.
  *
  * Copyright (C) 1999 Dave Cole
- * Copyright (C) 2003, 2006 Bas Wijnen <shevek@fmf.nl>
+ * Copyright (C) 2003-2007 Bas Wijnen <shevek@fmf.nl>
  * Copyright (C) 2005,2006 Roland Clobus <rclobus@bigfoot.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -37,52 +37,52 @@ static void player_setup(Player * player, int playernum,
 static Player *player_by_name(Game * game, char *name);
 
 
-static gint next_player_num(Game * game, gboolean force_viewer)
+/** Find a free number for a connecting player.
+ *  @param game The game
+ *  @param force_viewer The connecting player must be a viewer
+ */
+static gint next_free_player_num(Game * game, gboolean force_viewer)
 {
-	gint idx = game->params->num_players;
+	gint idx;
 
 	if (!force_viewer) {
 		GList *list;
-		gboolean players[MAX_PLAYERS];
-		memset(players, 0, sizeof(players));
+		gboolean player_taken[MAX_PLAYERS];
+		gint available = game->params->num_players;
+
+		memset(player_taken, 0, sizeof(player_taken));
 		playerlist_inc_use_count(game);
 		for (list = game->player_list;
 		     list != NULL; list = g_list_next(list)) {
 			Player *player = list->data;
 			if (player->num >= 0
 			    && !player_is_viewer(game, player->num)
-			    && !player->disconnected)
-				players[player->num] = TRUE;
+			    && (!player->disconnected
+				|| player->sm->use_cache)) {
+				player_taken[player->num] = TRUE;
+				--available;
+			}
 		}
 		playerlist_dec_use_count(game);
 
-		if (game->random_order) {
-			gint empty = 0, skip;
-			for (idx = 0; idx < game->params->num_players;
-			     idx++)
-				if (!players[idx])
-					empty++;
-			if (empty > 0) {
-				skip = get_rand(empty);
-				for (idx = 0;
-				     idx < game->params->num_players;
-				     idx++)
-					if (!players[idx] && skip-- == 0)
-						break;
+		if (available > 0) {
+			gint skip;
+			if (game->random_order) {
+				skip = get_rand(available);
 			} else {
-				idx = game->params->num_players;
+				skip = 0;
 			}
-		} else {
-			for (idx = 0; idx < game->params->num_players;
-			     idx++)
-				if (!players[idx])
-					break;
+			idx = 0;
+			while (player_taken[idx] || skip-- != 0)
+				++idx;
+			return idx;
 		}
 	}
-	if (idx == game->params->num_players)
-		while (player_by_num(game, idx) != NULL)
-			++idx;
 
+	/* No players available/wanted, look for a viewer number */
+	idx = game->params->num_players;
+	while (player_by_num(game, idx) != NULL)
+		++idx;
 	return idx;
 }
 
@@ -96,10 +96,10 @@ static gboolean mode_global(Player * player, gint event)
 	case SM_FREE:
 		if (player->name != NULL)
 			g_free(player->name);
+		if (player->style != NULL)
+			g_free(player->style);
 		if (player->location != NULL)
 			g_free(player->location);
-		if (player->client_version != NULL)
-			g_free(player->client_version);
 		if (player->devel != NULL)
 			deck_free(player->devel);
 		if (player->num >= 0
@@ -114,7 +114,8 @@ static gboolean mode_global(Player * player, gint event)
 	case SM_NET_CLOSE:
 		player_remove(player);
 		if (player->num >= 0) {
-			player_broadcast(player, PB_OTHERS, "has quit\n");
+			player_broadcast(player, PB_OTHERS, FIRST_VERSION,
+					 LATEST_VERSION, "has quit\n");
 			player_archive(player);
 		} else {
 			player_free(player);
@@ -124,25 +125,40 @@ static gboolean mode_global(Player * player, gint event)
 	case SM_RECV:
 		if (sm_recv(sm, "chat %S", &text)) {
 			if (strlen(text) > MAX_CHAT)
-				sm_send(sm, "ERR %s\n",
-					_("chat too long"));
+				player_send(player, FIRST_VERSION,
+					    LATEST_VERSION, "ERR %s\n",
+					    _("chat too long"));
 			else
 				player_broadcast(player, PB_ALL,
+						 FIRST_VERSION,
+						 LATEST_VERSION,
 						 "chat %s\n", text);
 			g_free(text);
 			return TRUE;
 		}
 		if (sm_recv(sm, "name %S", &text)) {
 			if (text[0] == '\0')
-				sm_send(sm, "ERR invalid-name\n");
+				player_send(player, FIRST_VERSION,
+					    LATEST_VERSION,
+					    "ERR invalid-name\n");
 			else if (strlen(text) > MAX_NAME_LENGTH)
-				sm_send(sm, "ERR %s\n",
-					_("name too long"));
+				player_send(player, FIRST_VERSION,
+					    LATEST_VERSION, "ERR %s\n",
+					    _("name too long"));
 			else
 				player_set_name(player, text);
 			g_free(text);
 			return TRUE;
 		}
+		if (sm_recv(sm, "style %S", &text)) {
+			if (player->style)
+				g_free(player->style);
+			player->style = text;
+			player_broadcast(player, PB_ALL, V0_11,
+					 LATEST_VERSION, "style %s\n",
+					 text);
+			return TRUE;
+		}
 		break;
 	default:
 		break;
@@ -158,8 +174,9 @@ static gboolean mode_unhandled(Player * player, gint event)
 	switch (event) {
 	case SM_RECV:
 		if (sm_recv(sm, "extension %S", &text)) {
-			sm_send(sm, "NOTE %s\n",
-				N_("ignoring unknown extension"));
+			player_send(player, FIRST_VERSION, LATEST_VERSION,
+				    "NOTE %s\n",
+				    N_("ignoring unknown extension"));
 			log_message(MSG_INFO,
 				    "ignoring unknown extension from %s: %s\n",
 				    player->name, text);
@@ -187,7 +204,8 @@ static gboolean tournament_start_cb(gpointer data)
 	if (game->num_players == game->params->num_players)
 		return FALSE;
 
-	player_broadcast(player, PB_SILENT, "NOTE %s\n",
+	player_broadcast(player, PB_SILENT, FIRST_VERSION, LATEST_VERSION,
+			 "NOTE %s\n",
 			 N_("Game starts, adding computer players"));
 	/* add computer players to start game */
 	for (i = game->num_players; i < game->params->num_players; i++) {
@@ -216,7 +234,7 @@ static gboolean talk_about_tournament_cb(gpointer data)
 	    N_("The game starts in %s minutes.") :
 	    N_("The game starts in %s minute.");
 
-	player_broadcast(player, PB_SILENT,
+	player_broadcast(player, PB_SILENT, FIRST_VERSION, LATEST_VERSION,
 			 "NOTE1 %d|%s\n",
 			 game->params->tournament_time, message);
 	game->params->tournament_time--;
@@ -294,6 +312,7 @@ Player *player_new(Game * game, int fd, gchar * location)
 	player->islands_discovered = 0;
 	player->disconnected = FALSE;
 	player->name = g_strdup(name);
+	player->style = g_strdup("square");
 	player->special_points = NULL;
 	player->special_points_next_id = 0;
 
@@ -306,8 +325,10 @@ Player *player_new(Game * game, int fd, gchar * location)
 			g_timeout_add(1000, &talk_about_tournament_cb,
 				      player);
 		} else {
-			sm_send_uncached(sm, "NOTE %s\n",
-					 N_("This game will start soon."));
+			player_send_uncached(player, FIRST_VERSION,
+					     LATEST_VERSION, "NOTE %s\n",
+					     N_
+					     ("This game will start soon."));
 		}
 	}
 
@@ -323,17 +344,19 @@ Player *player_new(Game * game, int fd, gchar * location)
 static void player_set_name_real(Player * player, gchar * name,
 				 gboolean public)
 {
-	StateMachine *sm = player->sm;
 	Game *game = player->game;
+	Player *player_temp;
 
 	g_assert(name[0] != 0);
 
-	if (player_by_name(game, name) != NULL) {
+	if (((player_temp = player_by_name(game, name)) != NULL) &&
+	    (player_temp != player)) {
 		/* make it a note, not an error, so nothing bad happens
 		 * (on error the AI would disconnect) */
-		sm_send(sm, "NOTE %s\n",
-			N_
-			("Name not changed: new name is already in use"));
+		player_send(player, FIRST_VERSION, LATEST_VERSION,
+			    "NOTE %s\n",
+			    N_
+			    ("Name not changed: new name is already in use"));
 		return;
 	}
 
@@ -343,7 +366,8 @@ static void player_set_name_real(Player * player, gchar * name,
 	}
 
 	if (public)
-		player_broadcast(player, PB_ALL, "is %s\n", player->name);
+		player_broadcast(player, PB_ALL, FIRST_VERSION,
+				 LATEST_VERSION, "is %s\n", player->name);
 
 	driver->player_renamed(player);
 	driver->player_change(game);
@@ -359,7 +383,7 @@ static void player_setup(Player * player, int playernum,
 
 	player->num = playernum;
 	if (player->num < 0) {
-		player->num = next_player_num(game, force_viewer);
+		player->num = next_free_player_num(game, force_viewer);
 	}
 
 	if (!player_is_viewer(game, player->num)) {
@@ -471,7 +495,8 @@ void player_archive(Player * player)
 			if (quote == NULL)
 				break;
 			quotelist_delete(game->quotes, quote);
-			player_broadcast(player, PB_RESPOND,
+			player_broadcast(player, PB_RESPOND, FIRST_VERSION,
+					 LATEST_VERSION,
 					 "domestic-quote delete %d\n",
 					 quote->var.d.quote_num);
 		}
@@ -480,6 +505,12 @@ void player_archive(Player * player)
 		trade_finish_domestic(player);
 	}
 
+	/* If the player was robbing something, auto-undo to robber
+	 * placement.  */
+	if (state == (StateFunc) mode_select_robbed
+	    || state == (StateFunc) mode_select_pirated)
+		robber_undo(player);
+
 	/* Mark the player as disconnected */
 	player->disconnected = TRUE;
 	game->num_players--;
@@ -505,7 +536,8 @@ void player_revive(Player * newp, char *name)
 		     current = g_list_next(current)) {
 			p = current->data;
 			if (!strcmp(name, p->name))
-				if (p->disconnected && p != newp)
+				if (p->disconnected && !p->sm->use_cache
+				    && p != newp)
 					break;
 		}
 		playerlist_dec_use_count(game);
@@ -516,7 +548,8 @@ void player_revive(Player * newp, char *name)
 		for (current = game->player_list; current != NULL;
 		     current = g_list_next(current)) {
 			p = current->data;
-			if (p->disconnected && p != newp)
+			if (p->disconnected && !p->sm->use_cache
+			    && p != newp)
 				break;
 		}
 		playerlist_dec_use_count(game);
@@ -609,7 +642,8 @@ void player_revive(Player * newp, char *name)
 	/* Make sure the name in the broadcast doesn't contain the separator */
 	safe_name = g_strdup(newp->name);
 	g_strdelimit(safe_name, "|", '_');
-	player_broadcast(newp, PB_SILENT, "NOTE1 %s|%s\n", safe_name,
+	player_broadcast(newp, PB_SILENT, FIRST_VERSION, LATEST_VERSION,
+			 "NOTE1 %s|%s\n", safe_name,
 			 /* %s is the name of the reconnecting player */
 			 N_("%s has reconnected."));
 	g_free(safe_name);
@@ -629,16 +663,18 @@ gboolean mode_viewer(Player * player, gint event)
 	/* first see if this is a valid event for this mode */
 	if (sm_recv(sm, "play")) {
 		/* try to be the first available player */
-		num = next_player_num(game, FALSE);
+		num = next_free_player_num(game, FALSE);
 		if (num >= game->params->num_players) {
-			sm_send(sm, "ERR game-full");
+			player_send(player, FIRST_VERSION, LATEST_VERSION,
+				    "ERR game-full");
 			return TRUE;
 		}
 	} else if (sm_recv(sm, "play %d", &num)) {
 		/* try to be the specified player number */
 		if (num >= game->params->num_players
 		    || !player_by_num(game, num)->disconnected) {
-			sm_send(sm, "ERR invalid-player");
+			player_send(player, FIRST_VERSION, LATEST_VERSION,
+				    "ERR invalid-player");
 			return TRUE;
 		}
 	} else
@@ -648,8 +684,9 @@ gboolean mode_viewer(Player * player, gint event)
 
 	other = player_by_num(game, num);
 	if (other == NULL) {
-		sm_send(sm, "Ok\n");
-		player_broadcast(player, PB_ALL, "was viewer %d\n",
+		player_send(player, FIRST_VERSION, LATEST_VERSION, "Ok\n");
+		player_broadcast(player, PB_ALL, FIRST_VERSION,
+				 LATEST_VERSION, "was viewer %d\n",
 				 player->num);
 		player_setup(player, player->num, player->name, FALSE);
 		sm_goto(sm, (StateFunc) mode_pre_game);
@@ -666,47 +703,81 @@ static gboolean mode_bad_version(Player * player, gint event)
 	sm_state_name(sm, "mode_bad_version");
 	switch (event) {
 	case SM_ENTER:
-		sm_send_uncached(sm, "ERR sorry, version conflict\n");
+		player_send_uncached(player, FIRST_VERSION, LATEST_VERSION,
+				     "ERR sorry, version conflict\n");
 		player_free(player);
 		break;
 	}
 	return FALSE;
 }
 
-static gboolean check_versions(gchar * client_version)
+/* Supported version names.  Whenever this list changes, the enum in server.h
+ * must also change.  */
+static const gchar *client_version_type_to_string(ClientVersionType cvt)
 {
-	return !strcmp(client_version, PROTOCOL_VERSION);
+	switch (cvt) {
+	case V0_10:
+		return "0.10";
+	case V0_11:
+		return "0.11";
+	}
+	g_return_val_if_reached("");
+}
+
+static gboolean check_versions(const gchar * client_version,
+			       Player * client)
+{
+	ClientVersionType i;
+	for (i = FIRST_VERSION; i <= LATEST_VERSION; ++i) {
+		if (strcmp
+		    (client_version,
+		     client_version_type_to_string(i)) == 0) {
+			client->version = i;
+			return TRUE;
+		}
+	}
+	return FALSE;
 }
 
 static gboolean mode_check_version(Player * player, gint event)
 {
 	StateMachine *sm = player->sm;
+	gchar *version;
 
 	sm_state_name(sm, "mode_check_version");
 	switch (event) {
 	case SM_ENTER:
-		sm_send_uncached(sm, "version report\n");
+		player_send_uncached(player, FIRST_VERSION, LATEST_VERSION,
+				     "version report\n");
 		break;
 
 	case SM_RECV:
-		if (sm_recv(sm, "version %S", &player->client_version)) {
-			if (check_versions(player->client_version)) {
+		if (sm_recv(sm, "version %S", &version)) {
+			gboolean result = check_versions(version, player);
+			if (result) {
 				sm_goto(sm, (StateFunc) mode_check_status);
 			} else {
+				/* PROTOCOL_VERSION is the current version
+				 * of the client when building this server.
+				 * Although it's not the only supported
+				 * version, it's the best the user can have.
+				 */
 				gchar *mismatch =
 				    g_strdup_printf("%s <-> %s",
 						    PROTOCOL_VERSION,
-						    player->
-						    client_version);
+						    version);
 				/* Make sure the argument does not contain the separator */
 				g_strdelimit(mismatch, "|", '_');
-				sm_send_uncached(sm, "NOTE1 %s|%s\n",
-						 mismatch,
-						 N_
-						 ("Version mismatch: %s"));
+				player_send_uncached(player, FIRST_VERSION,
+						     LATEST_VERSION,
+						     "NOTE1 %s|%s\n",
+						     mismatch,
+						     N_
+						     ("Version mismatch: %s"));
 				g_free(mismatch);
 				sm_goto(sm, (StateFunc) mode_bad_version);
 			}
+			g_free(version);
 			return TRUE;
 		}
 		break;
@@ -724,7 +795,8 @@ static gboolean mode_check_status(Player * player, gint event)
 	sm_state_name(sm, "mode_check_status");
 	switch (event) {
 	case SM_ENTER:
-		sm_send_uncached(sm, "status report\n");
+		player_send_uncached(player, FIRST_VERSION, LATEST_VERSION,
+				     "status report\n");
 		break;
 
 	case SM_RECV:
@@ -857,8 +929,9 @@ Player *player_none(Game * game)
 	return &player;
 }
 
-/* Broadcast a message to all players and viewers - prepend "player %d " to all
- * players except the one generating the message.
+/** Broadcast a message to all players and viewers - prepend "player %d " to
+ * all players except the one generating the message.
+ * Also prepend 'extension' when this message is a protocol extension.
  *
  *  send to  PB_SILENT PB_RESPOND PB_ALL PB_OTHERS
  *  player      -           -       +        **
@@ -867,32 +940,131 @@ Player *player_none(Game * game)
  * +  = prepend 'player %d' to the message
  * -  = don't alter the message
  */
-void player_broadcast(Player * player, BroadcastType type, const char *fmt,
-		      ...)
+static void player_broadcast_internal(Player * player, BroadcastType type,
+				      const gchar * message,
+				      gboolean is_extension,
+				      ClientVersionType
+				      first_supported_version,
+				      ClientVersionType
+				      last_supported_version)
 {
 	Game *game = player->game;
-	gchar *buff;
 	GList *list;
+
+	playerlist_inc_use_count(game);
+	for (list = game->player_list; list != NULL;
+	     list = g_list_next(list)) {
+		Player *scan = list->data;
+		if ((scan->disconnected && !scan->sm->use_cache) || scan->num < 0
+		    || scan->version < first_supported_version
+		    || scan->version > last_supported_version)
+			continue;
+		if (type == PB_SILENT
+		    || (scan == player && type == PB_RESPOND)) {
+			if (is_extension) {
+				player_send(scan, first_supported_version,
+					    last_supported_version,
+					    "extension %s", message);
+			} else {
+				player_send(scan, first_supported_version,
+					    last_supported_version, "%s",
+					    message);
+			}
+		} else if (scan != player || type == PB_ALL) {
+			if (is_extension) {
+				player_send(scan, first_supported_version,
+					    last_supported_version,
+					    "extension player %d %s",
+					    player->num, message);
+			} else {
+				player_send(scan, first_supported_version,
+					    last_supported_version,
+					    "player %d %s", player->num,
+					    message);
+			}
+
+		}
+	}
+	playerlist_dec_use_count(game);
+}
+
+/** As player_broadcast, but will add the 'extension' keyword */
+void player_broadcast_extension(Player * player, BroadcastType type,
+				ClientVersionType first_supported_version,
+				ClientVersionType last_supported_version,
+				const char *fmt, ...)
+{
+	gchar *buff;
 	va_list ap;
 
 	va_start(ap, fmt);
 	buff = sm_vformat(fmt, ap);
 	va_end(ap);
 
-	playerlist_inc_use_count(game);
-	for (list = game->player_list; list != NULL;
-	     list = g_list_next(list)) {
-		Player *scan = list->data;
-		if (scan->disconnected || scan->num < 0)
-			continue;
-		if (type == PB_SILENT
-		    || (scan == player && type == PB_RESPOND))
-			sm_write(scan->sm, buff);
-		else if (scan != player || type == PB_ALL)
-			sm_send(scan->sm, "player %d %s", player->num,
-				buff);
-	}
-	playerlist_dec_use_count(game);
+	player_broadcast_internal(player, type, buff, TRUE,
+				  first_supported_version,
+				  last_supported_version);
+	g_free(buff);
+}
+
+/** Broadcast a message to all players and viewers */
+void player_broadcast(Player * player, BroadcastType type,
+		      ClientVersionType first_supported_version,
+		      ClientVersionType last_supported_version,
+		      const char *fmt, ...)
+{
+	gchar *buff;
+	va_list ap;
+
+	va_start(ap, fmt);
+	buff = sm_vformat(fmt, ap);
+	va_end(ap);
+
+	player_broadcast_internal(player, type, buff, FALSE,
+				  first_supported_version,
+				  last_supported_version);
+	g_free(buff);
+}
+
+/** Send a message to one player */
+void player_send(Player * player,
+		 ClientVersionType first_supported_version,
+		 ClientVersionType last_supported_version, const char *fmt,
+		 ...)
+{
+	gchar *buff;
+	va_list ap;
+
+	if (player->version < first_supported_version
+	    || player->version > last_supported_version)
+		return;
+
+	va_start(ap, fmt);
+	buff = sm_vformat(fmt, ap);
+	va_end(ap);
+
+	sm_write(player->sm, buff);
+	g_free(buff);
+}
+
+/** Send a message to one player, even when caching is turned on */
+void player_send_uncached(Player * player,
+			  ClientVersionType first_supported_version,
+			  ClientVersionType last_supported_version,
+			  const char *fmt, ...)
+{
+	gchar *buff;
+	va_list ap;
+
+	if (player->version < first_supported_version
+	    || player->version > last_supported_version)
+		return;
+
+	va_start(ap, fmt);
+	buff = sm_vformat(fmt, ap);
+	va_end(ap);
+
+	sm_write_uncached(player->sm, buff);
 	g_free(buff);
 }
 
