@@ -27,7 +27,6 @@
 void trade_perform_maritime(Player * player,
 			    gint ratio, Resource supply, Resource receive)
 {
-	StateMachine *sm = player->sm;
 	Game *game = player->game;
 	Map *map = game->params->map;
 	gint check[NO_RESOURCE];
@@ -36,12 +35,14 @@ void trade_perform_maritime(Player * player,
 	if ((!game->rolled_dice) ||
 	    ((player->build_list != NULL || game->bought_develop) &&
 	     game->params->strict_trade)) {
-		sm_send(sm, "ERR wrong-time\n");
+		player_send(player, FIRST_VERSION, LATEST_VERSION,
+			    "ERR wrong-time\n");
 		return;
 	}
 
 	if (ratio < 2 || ratio > 4) {
-		sm_send(sm, "ERR trade bad-ratio\n");
+		player_send(player, FIRST_VERSION, LATEST_VERSION,
+			    "ERR trade bad-ratio\n");
 		return;
 	}
 
@@ -51,23 +52,23 @@ void trade_perform_maritime(Player * player,
 	switch (ratio) {
 	case 4:
 		if (player->assets[supply] < 4) {
-			sm_send(sm,
-				"ERR maritime trade, not 4 resources\n");
+			player_send(player, FIRST_VERSION, LATEST_VERSION,
+				    "ERR maritime trade, not 4 resources\n");
 			return;
 		}
 		break;
 	case 3:
 		if (!info.any_resource || player->assets[supply] < 3) {
-			sm_send(sm,
-				"ERR maritime trade, not 3 resources\n");
+			player_send(player, FIRST_VERSION, LATEST_VERSION,
+				    "ERR maritime trade, not 3 resources\n");
 			return;
 		}
 		break;
 	case 2:
 		if (!info.specific_resource[supply]
 		    || player->assets[supply] < 2) {
-			sm_send(sm,
-				"ERR maritime trade, not 2 resources\n");
+			player_send(player, FIRST_VERSION, LATEST_VERSION,
+				    "ERR maritime trade, not 2 resources\n");
 			return;
 		}
 		break;
@@ -83,7 +84,7 @@ void trade_perform_maritime(Player * player,
 	player->assets[supply] -= ratio;
 	player->assets[receive]++;
 
-	player_broadcast(player, PB_RESPOND,
+	player_broadcast(player, PB_RESPOND, FIRST_VERSION, LATEST_VERSION,
 			 "maritime-trade %d supply %r receive %r\n",
 			 ratio, supply, receive);
 }
@@ -135,12 +136,14 @@ gboolean mode_domestic_quote(Player * player, gint event)
 							player->num, -1);
 			if (quote == NULL)
 				break;
-			player_broadcast(player, PB_ALL,
+			player_broadcast(player, PB_ALL, FIRST_VERSION,
+					 LATEST_VERSION,
 					 "domestic-quote delete %d\n",
 					 quote->var.d.quote_num);
 			quotelist_delete(game->quotes, quote);
 		}
-		player_broadcast(player, PB_RESPOND,
+		player_broadcast(player, PB_RESPOND, FIRST_VERSION,
+				 LATEST_VERSION,
 				 "domestic-quote finish\n");
 
 		sm_goto(sm, (StateFunc) mode_wait_quote_exit);
@@ -154,11 +157,13 @@ gboolean mode_domestic_quote(Player * player, gint event)
 		quote = quotelist_find_domestic(game->quotes,
 						player->num, quote_num);
 		if (quote == NULL) {
-			sm_send(sm, "ERR quote already deleted\n");
+			player_send(player, FIRST_VERSION, LATEST_VERSION,
+				    "ERR quote already deleted\n");
 			return TRUE;
 		}
 		quotelist_delete(game->quotes, quote);
-		player_broadcast(player, PB_RESPOND,
+		player_broadcast(player, PB_RESPOND, FIRST_VERSION,
+				 LATEST_VERSION,
 				 "domestic-quote delete %d\n", quote_num);
 		return TRUE;
 	}
@@ -168,20 +173,23 @@ gboolean mode_domestic_quote(Player * player, gint event)
 		/* Make sure that quoting party can satisfy the trade
 		 */
 		if (!cost_can_afford(supply, player->assets)) {
-			sm_send(sm, "ERR quote not enough resources\n");
+			player_send(player, FIRST_VERSION, LATEST_VERSION,
+				    "ERR quote not enough resources\n");
 			return TRUE;
 		}
 		/* Make sure that the quote does not already exist
 		 */
 		if (quotelist_find_domestic
 		    (game->quotes, player->num, quote_num) != NULL) {
-			sm_send(sm, "INFO duplicate quote\n");
+			player_send(player, FIRST_VERSION, LATEST_VERSION,
+				    "INFO duplicate quote\n");
 			return TRUE;
 		}
 
 		quotelist_add_domestic(game->quotes, player->num,
 				       quote_num, supply, receive);
-		player_broadcast(player, PB_RESPOND,
+		player_broadcast(player, PB_RESPOND, FIRST_VERSION,
+				 LATEST_VERSION,
 				 "domestic-quote quote %d supply %R receive %R\n",
 				 quote_num, supply, receive);
 		return TRUE;
@@ -198,7 +206,8 @@ void trade_finish_domestic(Player * player)
 	Game *game = player->game;
 	GList *list;
 
-	player_broadcast(player, PB_RESPOND, "domestic-trade finish\n");
+	player_broadcast(player, PB_RESPOND, FIRST_VERSION, LATEST_VERSION,
+			 "domestic-trade finish\n");
 	sm_pop(sm);
 	for (list = player_first_real(game);
 	     list != NULL; list = player_next_real(list)) {
@@ -214,7 +223,6 @@ void trade_accept_domestic(Player * player,
 			   gint partner_num, gint quote_num,
 			   gint * supply, gint * receive)
 {
-	StateMachine *sm = player->sm;
 	Game *game = player->game;
 	QuoteInfo *quote;
 	Player *partner;
@@ -223,7 +231,8 @@ void trade_accept_domestic(Player * player,
 	if ((!game->rolled_dice) ||
 	    (((player->build_list != NULL) || (game->bought_develop)) &&
 	     (game->params->strict_trade))) {
-		sm_send(sm, "ERR wrong-time\n");
+		player_send(player, FIRST_VERSION, LATEST_VERSION,
+			    "ERR wrong-time\n");
 		return;
 	}
 
@@ -232,18 +241,21 @@ void trade_accept_domestic(Player * player,
 	quote =
 	    quotelist_find_domestic(game->quotes, partner_num, quote_num);
 	if (quote == NULL) {
-		sm_send(sm, "ERR quote not found\n");
+		player_send(player, FIRST_VERSION, LATEST_VERSION,
+			    "ERR quote not found\n");
 		return;
 	}
 	/* Make sure that both parties can satisfy the trade
 	 */
 	if (!cost_can_afford(quote->var.d.receive, player->assets)) {
-		sm_send(sm, "ERR quote cannot afford\n");
+		player_send(player, FIRST_VERSION, LATEST_VERSION,
+			    "ERR quote cannot afford\n");
 		return;
 	}
 	partner = player_by_num(game, partner_num);
 	if (!cost_can_afford(quote->var.d.supply, partner->assets)) {
-		sm_send(sm, "ERR quote partner cannot afford\n");
+		player_send(player, FIRST_VERSION, LATEST_VERSION,
+			    "ERR quote partner cannot afford\n");
 		return;
 	}
 
@@ -254,7 +266,7 @@ void trade_accept_domestic(Player * player,
 	cost_refund(quote->var.d.supply, player->assets);
 	cost_buy(quote->var.d.receive, player->assets);
 
-	player_broadcast(player, PB_RESPOND,
+	player_broadcast(player, PB_RESPOND, FIRST_VERSION, LATEST_VERSION,
 			 "domestic-trade accept player %d quote %d supply %R receive %R\n",
 			 partner_num, quote_num, supply, receive);
 
@@ -271,7 +283,8 @@ void trade_accept_domestic(Player * player,
 
 		quote = quotelist_next(quote);
 		if (!cost_can_afford(tmp->var.d.supply, partner->assets)) {
-			player_broadcast(partner, PB_ALL,
+			player_broadcast(partner, PB_ALL, FIRST_VERSION,
+					 LATEST_VERSION,
 					 "domestic-quote delete %d\n",
 					 tmp->var.d.quote_num);
 			quotelist_delete(game->quotes, tmp);
@@ -291,7 +304,7 @@ static void process_call_domestic(Player * player, gint * supply,
 		game->quote_receive[i] = receive[i];
 	}
 
-	player_broadcast(player, PB_RESPOND,
+	player_broadcast(player, PB_RESPOND, FIRST_VERSION, LATEST_VERSION,
 			 "domestic-trade call supply %R receive %R\n",
 			 supply, receive);
 	/* make sure all the others are back in quote mode.  They may have
@@ -309,7 +322,6 @@ static void process_call_domestic(Player * player, gint * supply,
 
 static void call_domestic(Player * player, gint * supply, gint * receive)
 {
-	StateMachine *sm = player->sm;
 	Game *game = player->game;
 	Player *partner;
 	gint num_supply, num_receive;
@@ -322,8 +334,9 @@ static void call_domestic(Player * player, gint * supply, gint * receive)
 	for (idx = 0; idx < NO_RESOURCE; idx++) {
 		if (supply[idx]) {
 			if (player->assets[idx] == 0) {
-				sm_send(sm,
-					"ERR not enough resources for this quote\n");
+				player_send(player, FIRST_VERSION,
+					    LATEST_VERSION,
+					    "ERR not enough resources for this quote\n");
 				return;
 			}
 			num_supply++;
@@ -332,7 +345,8 @@ static void call_domestic(Player * player, gint * supply, gint * receive)
 			++num_receive;
 	}
 	if (num_supply == 0 && num_receive == 0) {
-		sm_send(sm, "ERR empty quote\n");
+		player_send(player, FIRST_VERSION, LATEST_VERSION,
+			    "ERR empty quote\n");
 		return;
 	}
 	quote = quotelist_first(game->quotes);
@@ -354,7 +368,8 @@ static void call_domestic(Player * player, gint * supply, gint * receive)
 		if (idx < NO_RESOURCE) {
 			partner =
 			    player_by_num(game, curr->var.d.player_num);
-			player_broadcast(partner, PB_ALL,
+			player_broadcast(partner, PB_ALL, FIRST_VERSION,
+					 LATEST_VERSION,
 					 "domestic-quote delete %d\n",
 					 curr->var.d.quote_num);
 			quotelist_delete(game->quotes, curr);
